library ucDlpLib;

{ -------------------------------------------------------------------------}
{ ucdlplib.dpr : Delphi add-on library source code.                        }
{ Compiles to ucdlplib.dll.  For use with demo program.                    }
{ Designed to work with Delphi 4.0 or above.                               }
{                                                                          }
{ Redistribution of this file is not allowed.                              }
{                                                                          }
{ UCalc Fast Math Parser (Beta J)                                          }
{ Copyright (c) 1998-2004 by Daniel Corbier.  All rights reserved.         }
{ http://www.ucalc.com/mathparser                                          }
{ -------------------------------------------------------------------------}


uses
   SysUtils,
   Classes,
   Math,
   Mask,
   Windows,
   FileCtrl;

{$DEFINE ucAddOnLibrary}
{$INCLUDE ucalc}


function uc_PreParse_StringArraySyntax(var t: ucThreadType): Longint; stdcall;
var
   Expr: string;
   StringVar: string;
   CharPosArg: string;
   OpenBracket: Longint;
begin
   // For things like MyString[x]
   if t.ErrorNumber = ucErr_SyntaxError then
   begin
      if ucErrorData(0, t.ucThread) = '[' then
      begin
         Expr := ucGetExpr(t);
         OpenBracket := Pos('[', Expr);
         StringVar := Copy(Expr, 1, OpenBracket-1);
         CharPosArg := Copy(Expr, OpenBracket+1, Length(Expr)-OpenBracket-1);

         ucSetExpr('COPY('+StringVar+','+CharPosArg+', 1)', t);
      end;
   end;
end;

function uc_Floor(var t: ucThreadType): Extended; stdcall;
begin
   uc_Floor := Floor(ucParam(1, t));
end;

function uc_Frac(var t: ucThreadType): Extended; stdcall;
begin
   uc_Frac := Frac(ucParam(1, t));
end;

function uc_LnXP1(var t: ucThreadType): Extended; stdcall;
begin
   uc_LnXP1 := LnXP1(ucParam(1, t));
end;

function uc_LogN(var t: ucThreadType): Extended; stdcall;
begin
   uc_LogN := LogN(ucParam(1, t), ucParam(2, t));
end;

function uc_Pi(var t: ucThreadType): Extended; stdcall;
begin
   uc_Pi := Pi;
end;

function uc_Poly(var t: ucThreadType): Extended; stdcall;
var
   x: Extended;
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   CoefficientArray: array of Double;
begin
   x := ucParam(1, t);
   ArrayIndex   := Trunc(ucParam(2, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(CoefficientArray, UpperBound-LowerBound+1);
   CopyMemory(CoefficientArray, ArrayAddress, 8*Length(CoefficientArray));

   uc_Poly := Poly(x, CoefficientArray);
end;

function uc_Round(var t: ucThreadType): Extended; stdcall;
begin
   uc_Round := Round(ucParam(1, t));
end;

function uc_Trunc(var t: ucThreadType): Extended; stdcall;
begin
   uc_Trunc := Trunc(ucParam(1, t));
end;

function uc_DoubleDecliningBalance(var t: ucThreadType): Extended; stdcall;
var
   Cost, Salvage: Extended;
   Life, Period: Integer;
begin
   Cost    := ucParam(1, t);
   Salvage := ucParam(2, t);
   Life    := Trunc(ucParam(3, t));
   Period  := Trunc(ucParam(4, t));
   uc_DoubleDecliningBalance := DoubleDecliningBalance(Cost, Salvage, Life, Period);
end;

function uc_FutureValue(var t: ucThreadType): Extended; stdcall;
var
   Rate, Payment, PresentValue: Extended;
   NPeriods: Integer;
   PaymentTime: TPaymentTime;
begin
   Rate         := ucParam(1, t);
   NPeriods     := Trunc(ucParam(2, t));
   Payment      := ucParam(3, t);
   PresentValue := ucParam(4, t);
   PaymentTime  := TPaymentTime(Trunc(ucParam(5, t)));

   uc_FutureValue := FutureValue(Rate, NPeriods, Payment, PresentValue, PaymentTime);
end;

function uc_InterestPayment(var t: ucThreadType): Extended; stdcall;
var
   Rate: Extended;
   Period, NPeriods: Integer;
   PresentValue, FutureValue: Extended;
   PaymentTime: TPaymentTime;
begin
   Rate         := ucParam(1, t);
   Period       := Trunc(ucParam(2, t));
   NPeriods     := Trunc(ucParam(3, t));
   PresentValue := ucParam(4, t);
   FutureValue  := ucParam(5, t);
   PaymentTime  := TPaymentTime(Trunc(ucParam(6, t)));

   uc_InterestPayment := InterestPayment(Rate, Period, NPeriods, PresentValue, FutureValue, PaymentTime);
end;

function uc_InterestRate(var t: ucThreadType): Extended; stdcall;
var
   NPeriods: Integer;
   Payment, PresentValue, FutureValue: Extended;
   PaymentTime: TPaymentTime;
begin
   NPeriods     := Trunc(ucParam(1, t));
   Payment      := ucParam(2, t);
   PresentValue := ucParam(3, t);
   FutureValue  := ucParam(4, t);
   PaymentTime  := TPaymentTime(Trunc(ucParam(5, t)));

   uc_InterestRate := InterestRate(NPeriods,Payment,PresentValue,FutureValue,PaymentTime);
end;

function uc_InternalRateOfReturn(var t: ucThreadType): Extended; stdcall;
var
   Guess: Extended;
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   CashFlows: array of Double;
begin
   Guess        := ucParam(1, t);
   ArrayIndex   := Trunc(ucParam(2, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(CashFlows, UpperBound-LowerBound+1);
   CopyMemory(CashFlows, ArrayAddress, 8*Length(CashFlows));

   uc_InternalRateOfReturn := InternalRateOfReturn(Guess, CashFlows);
end;

function uc_NetPresentValue(var t: ucThreadType): Extended; stdcall;
var
   Rate: Extended;
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   CashFlows: array of Double;
   PaymentTime: TPaymentTime;
begin
   Rate         := ucParam(1, t);
   ArrayIndex   := Trunc(ucParam(2, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);
   PaymentTime  := TPaymentTime(Trunc(ucParam(5, t)));

   SetLength(CashFlows, UpperBound-LowerBound+1);
   CopyMemory(CashFlows, ArrayAddress, 8*Length(CashFlows));

   uc_NetPresentValue := NetPresentValue(Rate, CashFlows, PaymentTime);
end;

function uc_NumberOfPeriods(var t: ucThreadType): Extended; stdcall;
var
   Rate, Payment, PresentValue, FutureValue: Extended;
   PaymentTime: TPaymentTime;
begin
   Rate         := ucParam(1, t);
   Payment      := ucParam(2, t);
   PresentValue := ucParam(3, t);
   FutureValue  := ucParam(4, t);
   PaymentTime  := TPaymentTime(Trunc(ucParam(5, t)));

   uc_NumberOfPeriods := NumberOfPeriods(Rate,Payment,PresentValue,FutureValue,PaymentTime);
end;

function uc_Payment(var t: ucThreadType): Extended; stdcall;
var
   Rate: Extended;
   NPeriods: Integer;
   PresentValue, FutureValue: Extended;
   PaymentTime: TPaymentTime;
begin
   Rate         := ucParam(1, t);
   NPeriods     := Trunc(ucParam(2, t));
   PresentValue := ucParam(3, t);
   FutureValue  := ucParam(4, t);
   PaymentTime  := TPaymentTime(Trunc(ucParam(5, t)));

   uc_Payment := Payment(Rate,NPeriods,PresentValue,FutureValue,PaymentTime);
end;

function uc_PeriodPayment(var t: ucThreadType): Extended; stdcall;
var
   Rate: Extended;
   Period, NPeriods: Integer;
   PresentValue, FutureValue: Extended;
   PaymentTime: TPaymentTime;
begin
   Rate         := ucParam(1, t);
   Period       := Trunc(ucParam(2, t));
   NPeriods     := Trunc(ucParam(3, t));
   PresentValue := ucParam(4, t);
   FutureValue  := ucParam(5, t);
   PaymentTime  := TPaymentTime(Trunc(ucParam(6, t)));

   uc_PeriodPayment := PeriodPayment(Rate,Period,NPeriods,PresentValue,FutureValue,PaymentTime);
end;

function uc_PresentValue(var t: ucThreadType): Extended; stdcall;
var
   Rate: Extended;
   NPeriods: Integer;
   Payment, FutureValue: Extended;
   PaymentTime: TPaymentTime;
begin
   Rate        := ucParam(1, t);
   NPeriods    := Trunc(ucParam(2, t));
   Payment     := ucParam(3, t);
   FutureValue := ucParam(4, t);
   PaymentTime := TPaymentTime(Trunc(ucParam(5, t)));

   uc_PresentValue := PresentValue(Rate,NPeriods,Payment,FutureValue,PaymentTime);
end;

function uc_SLNDepreciation(var t: ucThreadType): Extended; stdcall;
begin
   uc_SLNDepreciation := SLNDepreciation(ucParam(1, t), ucParam(2, t), Trunc(ucParam(3, t)));
end;

function uc_SYDDepreciation(var t: ucThreadType): Extended; stdcall;
begin
   uc_SYDDepreciation := SYDDepreciation(ucParam(1, t), ucParam(2, t),
                         Trunc(ucParam(3, t)), Trunc(ucParam(4, t)));
end;

function uc_RandG(var t: ucThreadType): Extended; stdcall;
begin
   uc_RandG := RandG(ucParam(1, t), ucParam(2, t));
end;

function uc_Random(var t: ucThreadType): Extended; stdcall;
begin
   uc_Random := Random(Trunc(ucParam(1, t)));
end;

function uc_MaxIntValue(var t: ucThreadType): Extended; stdcall;
var
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   ArrayData: array of Integer;
begin
   ArrayIndex   := Trunc(ucParam(1, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(ArrayData, UpperBound-LowerBound+1);
   CopyMemory(ArrayData, ArrayAddress, 4*Length(ArrayData));

   uc_MaxIntValue := MaxIntValue(ArrayData);
end;

function uc_MaxValue(var t: ucThreadType): Extended; stdcall;
var
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   ArrayData: array of Double;
begin
   ArrayIndex   := Trunc(ucParam(1, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(ArrayData, UpperBound-LowerBound+1);
   CopyMemory(ArrayData, ArrayAddress, 8*Length(ArrayData));

   uc_MaxValue := MaxValue(ArrayData);
end;

function uc_Mean(var t: ucThreadType): Extended; stdcall;
var
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   ArrayData: array of Double;
begin
   ArrayIndex   := Trunc(ucParam(1, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(ArrayData, UpperBound-LowerBound+1);
   CopyMemory(ArrayData, ArrayAddress, 8*Length(ArrayData));

   uc_Mean := Mean(ArrayData);
end;

function uc_MinIntValue(var t: ucThreadType): Extended; stdcall;
var
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   ArrayData: array of Integer;
begin
   ArrayIndex   := Trunc(ucParam(1, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(ArrayData, UpperBound-LowerBound+1);
   CopyMemory(ArrayData, ArrayAddress, 4*Length(ArrayData));

   uc_MinIntValue := MinIntValue(ArrayData);
end;

function uc_MinValue(var t: ucThreadType): Extended; stdcall;
var
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   ArrayData: array of Double;
begin
   ArrayIndex   := Trunc(ucParam(1, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(ArrayData, UpperBound-LowerBound+1);
   CopyMemory(ArrayData, ArrayAddress, 8*Length(ArrayData));

   uc_MinValue := MinValue(ArrayData);
end;

function uc_Norm(var t: ucThreadType): Extended; stdcall;
var
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   ArrayData: array of Double;
begin
   ArrayIndex   := Trunc(ucParam(1, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(ArrayData, UpperBound-LowerBound+1);
   CopyMemory(ArrayData, ArrayAddress, 8*Length(ArrayData));

   uc_Norm := Norm(ArrayData);
end;

function uc_PopnStdDev(var t: ucThreadType): Extended; stdcall;
var
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   ArrayData: array of Double;
begin
   ArrayIndex   := Trunc(ucParam(1, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(ArrayData, UpperBound-LowerBound+1);
   CopyMemory(ArrayData, ArrayAddress, 8*Length(ArrayData));

   uc_PopnStdDev := PopnStdDev(ArrayData);
end;

function uc_PopnVariance(var t: ucThreadType): Extended; stdcall;
var
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   ArrayData: array of Double;
begin
   ArrayIndex   := Trunc(ucParam(1, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(ArrayData, UpperBound-LowerBound+1);
   CopyMemory(ArrayData, ArrayAddress, 8*Length(ArrayData));

   uc_PopnVariance := PopnVariance(ArrayData);
end;

function uc_StdDev(var t: ucThreadType): Extended; stdcall;
var
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   ArrayData: array of Double;
begin
   ArrayIndex   := Trunc(ucParam(1, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(ArrayData, UpperBound-LowerBound+1);
   CopyMemory(ArrayData, ArrayAddress, 8*Length(ArrayData));

   uc_StdDev := StdDev(ArrayData);
end;

function uc_Sum(var t: ucThreadType): Extended; stdcall;
var
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   ArrayData: array of Double;
begin
   ArrayIndex   := Trunc(ucParam(1, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(ArrayData, UpperBound-LowerBound+1);
   CopyMemory(ArrayData, ArrayAddress, 8*Length(ArrayData));

   uc_Sum := Sum(ArrayData);
end;

function uc_SumInt(var t: ucThreadType): Extended; stdcall;
var
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   ArrayData: array of Integer;
begin
   ArrayIndex   := Trunc(ucParam(1, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(ArrayData, UpperBound-LowerBound+1);
   CopyMemory(ArrayData, ArrayAddress, 4*Length(ArrayData));

   uc_SumInt := SumInt(ArrayData);
end;

function uc_SumOfSquares(var t: ucThreadType): Extended; stdcall;
var
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   ArrayData: array of Double;
begin
   ArrayIndex   := Trunc(ucParam(1, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(ArrayData, UpperBound-LowerBound+1);
   CopyMemory(ArrayData, ArrayAddress, 8*Length(ArrayData));

   uc_SumOfSquares := SumOfSquares(ArrayData);
end;

function uc_TotalVariance(var t: ucThreadType): Extended; stdcall;
var
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   ArrayData: array of Double;
begin
   ArrayIndex   := Trunc(ucParam(1, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(ArrayData, UpperBound-LowerBound+1);
   CopyMemory(ArrayData, ArrayAddress, 8*Length(ArrayData));

   uc_TotalVariance := TotalVariance(ArrayData);
end;

function uc_Variance(var t: ucThreadType): Extended; stdcall;
var
   ArrayAddress: Pointer;
   ArrayIndex, LowerBound, UpperBound: Longint;
   ArrayData: array of Double;
begin
   ArrayIndex   := Trunc(ucParam(1, t));
   ArrayAddress := Pointer(ucArrayInfo(ArrayIndex, ucArray_DataAddress));
   LowerBound   := ucLowerBound(ArrayIndex);
   UpperBound   := ucUpperBound(ArrayIndex);

   SetLength(ArrayData, UpperBound-LowerBound+1);
   CopyMemory(ArrayData, ArrayAddress, 8*Length(ArrayData));

   uc_Variance := Variance(ArrayData);
end;

function uc_ArcTan2(var t: ucThreadType): Extended; stdcall;
begin
   uc_ArcTan2 := ArcTan2(ucParam(1, t), ucParam(2, t));
end;

function uc_Date(var t: ucThreadType): Extended; stdcall;
begin
   uc_Date := Date();
end;

function uc_DateTimeToFileDate(var t: ucThreadType): Extended; stdcall;
begin
   uc_DateTimeToFileDate := DateTimeToFileDate(ucParam(1, t));
end;

function uc_DateTimeToStr(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(DateTimeToStr(ucParam(1, t)), t);
end;

function uc_DateToStr(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(DateToStr(ucParam(1, t)), t);
end;

function uc_DayOfWeek(var t: ucThreadType): Extended; stdcall;
begin
   uc_DayOfWeek := DayOfWeek(ucParam(1, t));
end;

function uc_EncodeDate(var t: ucThreadType): Extended; stdcall;
begin
   uc_EncodeDate := EncodeDate(trunc(ucParam(1, t)), Trunc(ucParam(2, t)), Trunc(ucParam(3, t)));
end;

function uc_EncodeTime(var t: ucThreadType): Extended; stdcall;
begin
   uc_EncodeTime := EncodeTime(Trunc(ucParam(1, t)), Trunc(ucParam(2, t)), Trunc(ucParam(3, t)), Trunc(ucParam(4, t)));
end;

function uc_FormatDateTime(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(FormatDateTime(ucParamStr(1, t), ucParam(2, t)), t);
end;

function uc_IncMonth(var t: ucThreadType): Extended; stdcall;
begin
   uc_IncMonth := IncMonth(ucParam(1, t), Trunc(ucParam(2, t)));
end;

function uc_IsLeapYear(var t: ucThreadType): Extended; stdcall;
begin
   if IsLeapYear(Trunc(ucParam(1, t))) = True then
      uc_IsLeapYear := 1  //+++
   else
      uc_IsLeapYear := 0;
end;

function uc_Now(var t: ucThreadType): Extended; stdcall;
begin
   uc_Now := Now;
end;

function uc_StrToDate(var t: ucThreadType): Extended; stdcall;
begin
   uc_StrToDate := StrToDate(ucParamStr(1, t));
end;

function uc_StrToDateTime(var t: ucThreadType): Extended; stdcall;
begin
   uc_StrToDateTime := StrToDateTime(ucParamStr(1, t));
end;

function uc_StrToTime(var t: ucThreadType): Extended; stdcall;
begin
   uc_StrToTime := StrToTime(ucParamStr(1, t));
end;

function uc_Time(var t: ucThreadType): Extended; stdcall;
begin
   uc_Time := Time;
end;

function uc_TimeToStr(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(TimeToStr(ucParam(1, t)), t);
end;

function uc_FloatToStr(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(FloatToStr(ucParam(1, t)), t);
end;

function uc_FloatToStrF(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(FloatToStrF(ucParam(1, t), TFloatFormat(Trunc(ucParam(2, t))),
               Trunc(ucParam(3, t)), Trunc(ucParam(4, t))), t);
end;

function uc_FormatFloat(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(FormatFloat(ucParamStr(1, t), ucParam(2, t)), t);
end;

function uc_StrToCurr(var t: ucThreadType): Extended; stdcall;
begin
   uc_StrToCurr := StrToCurr(ucParamStr(1, t));
end;

function uc_StrToFloat(var t: ucThreadType): Extended; stdcall;
begin
   uc_StrToFloat := StrToFloat(ucParamStr(1, t));
end;

function uc_AnsiCompareFileName(var t: ucThreadType): Extended; stdcall;
begin
   uc_AnsiCompareFileName := AnsiCompareFileName(ucParamStr(1, t), ucParamStr(2, t));
end;

function uc_AnsiLowerCaseFileName(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(AnsiLowerCaseFileName(ucParamStr(1, t)), t);
end;

function uc_AnsiUpperCaseFileName(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(AnsiUpperCaseFileName(ucParamStr(1, t)), t);
end;

function uc_ByteToCharIndex(var t: ucThreadType): Extended; stdcall;
begin
   uc_ByteToCharIndex := ByteToCharIndex(ucParamStr(1, t), Trunc(ucParam(2, t)));
end;

function uc_ByteToCharLen(var t: ucThreadType): Extended; stdcall;
begin
   uc_ByteToCharLen := ByteToCharLen(ucParamStr(1, t), Trunc(ucParam(2, t)));
end;

function uc_CharToByteIndex(var t: ucThreadType): Extended; stdcall;
begin
   uc_CharToByteIndex := CharToByteIndex(ucParamStr(1, t), Trunc(ucParam(2, t)));
end;

function uc_CharToByteLen(var t: ucThreadType): Extended; stdcall;
begin
   uc_CharToByteLen := CharToByteLen(ucParamStr(1, t), Trunc(ucParam(2, t)));
end;

function uc_CurrToStr(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(CurrToStr(ucParam(1, t)), t);
end;

function uc_CurrToStrF(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(CurrToStrF(ucParam(1, t), TFloatFormat(Trunc(ucParam(2, t))),
               Trunc(ucParam(3, t))) , t);
end;

function uc_FormatCurr(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(FormatCurr(ucParamStr(1, t), ucParam(2, t)), t);
end;

function uc_IntToHex(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(IntToHex(Trunc(ucParam(1, t)), Trunc(ucParam(2, t))), t);
end;

function uc_IntToStr(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(IntToStr(Trunc(ucParam(1, t))), t);
end;

function uc_AdjustLineBreaks(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(AdjustLineBreaks(ucParamStr(1, t)), t);
end;

function uc_AnsiCompareStr(var t: ucThreadType): Extended; stdcall;
begin
   uc_AnsiCompareStr := AnsiCompareStr(ucParamStr(1, t), ucParamStr(2, t));
end;

function uc_AnsiCompareText(var t: ucThreadType): Extended; stdcall;
begin
   uc_AnsiCompareText := AnsiCompareText(ucParamStr(1, t), ucParamStr(1, t));
end;

function uc_AnsiLowerCase(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(AnsiLowerCase(ucParamStr(1, t)), t);
end;

function uc_AnsiPos(var t: ucThreadType): Extended; stdcall;
begin
   uc_AnsiPos := AnsiPos(ucParamStr(1, t), ucParamStr(2, t));
end;

function uc_AnsiQuotedStr(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(AnsiQuotedStr(ucParamStr(1, t), Char(Trunc(ucParam(2, t)))), t);
end;

function uc_AnsiUpperCase(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(AnsiUpperCase(ucParamStr(1, t)), t);
end;

function uc_CompareStr(var t: ucThreadType): Extended; stdcall;
begin
   uc_CompareStr := CompareStr(ucParamStr(1, t), ucParamStr(2, t));
end;

function uc_CompareText(var t: ucThreadType): Extended; stdcall;
begin
   uc_CompareText := CompareText(ucParamStr(1, t), ucParamStr(2, t));
end;

function uc_Copy(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(Copy(ucParamStr(1, t), Trunc(ucParam(2, t)), Trunc(ucParam(3, t))), t);
end;

function uc_IsDelimiter(var t: ucThreadType): Extended; stdcall;
begin
   // +++ True -1 or 1 ?
   uc_IsDelimiter := Integer(IsDelimiter(ucParamStr(1, t), ucParamStr(2, t), Trunc(ucParam(3, t))));
end;

function uc_IsPathDelimiter(var t: ucThreadType): Extended; stdcall;
begin
   uc_IsPathDelimiter := Integer(IsPathDelimiter(ucParamStr(1, t), Trunc(ucParam(2, t))));
end;

function uc_LastDelimiter(var t: ucThreadType): Extended; stdcall;
begin
   uc_LastDelimiter := LastDelimiter(ucParamStr(1, t), ucParamStr(2, t));
end;

function uc_Length(var t: ucThreadType): Extended; stdcall;
begin
   uc_Length := Length(ucParamStr(1, t));
end;

function uc_LowerCase(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(ucParamStr(1, t), t);
end;

function uc_Pos(var t: ucThreadType): Extended; stdcall;
begin
   uc_Pos := Pos(ucParamStr(1, t), ucParamStr(2, t));
end;

function uc_QuotedStr(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(QuotedStr(ucParamStr(1, t)), t);
end;

function uc_StringOfChar(var t: ucThreadType): Extended; stdcall;
begin
   // +++ all Char arguments need to be overloaded
   ucReturnStr(StringOfChar(Char(Trunc(ucParam(1, t))), Trunc(ucParam(2, t))), t);
end;

function uc_Trim(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(Trim(ucParamStr(1, t)), t);
end;

function uc_TrimLeft(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(TrimLeft(ucParamStr(1, t)), t);
end;

function uc_TrimRight(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(TrimRight(ucParamStr(1, t)), t);
end;

function uc_UpperCase(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(UpperCase(ucParamStr(1, t)), t);
end;

function uc_AnsiStrComp(var t: ucThreadType): Extended; stdcall;
begin
   uc_AnsiStrComp := AnsiStrComp(ucParamStr(1, t), ucParamStr(2, t));
end;

function uc_AnsiStrIComp(var t: ucThreadType): Extended; stdcall;
begin
   uc_AnsiStrIComp := AnsiStrIComp(ucParamStr(1, t), ucParamStr(2, t));
end;

function uc_AnsiStrLComp(var t: ucThreadType): Extended; stdcall;
begin
   uc_AnsiStrLComp := AnsiStrLComp(ucParamStr(1, t), ucParamStr(2, t), Trunc(ucParam(3, t)));
end;

function uc_AnsiStrLIComp(var t: ucThreadType): Extended; stdcall;
begin
   uc_AnsiStrLIComp := AnsiStrLIComp(ucParamStr(1, t), ucParamStr(2, t), Trunc(ucParam(3, t)));
end;

function uc_AnsiStrLower(var t: ucThreadType): Extended; stdcall;
begin
   //++
   ucReturnStr(AnsiStrLower(ucParamStr(1, t)), t);
end;

function uc_AnsiStrPos(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(AnsiStrPos(ucParamStr(1, t), ucParamStr(2, t)), t);
end;

function uc_AnsiStrScan(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(AnsiStrScan(ucParamStr(1, t), Char(Trunc(ucParam(2, t)))), t);
end;

function uc_AnsiStrUpper(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(AnsiStrUpper(ucParamStr(1, t)), t);
end;

function uc_LineStart(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(LineStart(ucParamStr(1, t), ucParamStr(2, t)), t);
end;

function uc_StrCat(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(StrCat(ucParamStr(1, t), ucParamStr(2, t)), t);
end;

function uc_StrComp(var t: ucThreadType): Extended; stdcall;
begin
   uc_StrComp := StrComp(ucParamStr(1, t), ucParamStr(2, t));
end;

function uc_StrCopy(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(StrCopy(ucParamStr(1, t), ucParamStr(2, t)), t);
end;

function uc_StrIComp(var t: ucThreadType): Extended; stdcall;
begin
   uc_StrIComp := StrIComp(ucParamStr(1, t), ucParamStr(2, t));
end;

function uc_StrLCat(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(StrLCat(ucParamStr(1, t), ucParamStr(2, t), Trunc(ucParam(3, t))), t);
end;

function uc_StrLComp(var t: ucThreadType): Extended; stdcall;
begin
   uc_StrLComp := StrLComp(ucParamStr(1, t), ucParamStr(2, t), Trunc(ucParam(3, t)));
end;

function uc_StrLCopy(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(StrLCopy(ucParamStr(1, t), ucParamStr(2, t), Trunc(ucParam(3, t))), t);
end;

function uc_StrLen(var t: ucThreadType): Extended; stdcall;
begin
   uc_StrLen := StrLen(ucParamStr(1, t));
end;

function uc_StrLIComp(var t: ucThreadType): Extended; stdcall;
begin
   uc_StrLIComp := StrLIComp(ucParamStr(1, t), ucParamStr(2, t), Trunc(ucParam(3, t)));
end;

function uc_StrLower(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(StrLower(ucParamStr(1, t)), t);
end;

function uc_StrMove(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(StrMove(ucParamStr(1, t), ucParamStr(2, t), Trunc(ucParam(3, t))), t);
end;

function uc_StrPCopy(var t: ucThreadType): Extended; stdcall;
begin
   //+++
   ucReturnStr(StrPCopy(ucParamStr(1, t), ucParamStr(2, t)), t);
end;

function uc_StrPLCopy(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(StrPLCopy(ucParamStr(1, t), ucParamStr(2, t), Trunc(ucParam(3, t))), t);
end;

function uc_StrPos(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(StrPos(ucParamStr(1, t), ucParamStr(2, t)), t);
end;

function uc_StrRScan(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(StrRScan(ucParamStr(1, t), Char(Trunc(ucParam(2, t)))), t);
end;

function uc_StrScan(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(StrScan(ucParamStr(1, t), Char(Trunc(ucParam(2, t)))), t);
end;

function uc_StrUpper(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(StrUpper(ucParamStr(1, t)), t);
end;

function uc_StrToInt(var t: ucThreadType): Extended; stdcall;
begin
   uc_StrToInt := StrToInt(ucParamStr(1, t));
end;

function uc_StrToInt64(var t: ucThreadType): Extended; stdcall;
begin
   uc_StrToInt64 := StrToInt64(ucParamStr(1, t));
end;

function uc_StrToInt64Def(var t: ucThreadType): Extended; stdcall;
begin
   uc_StrToInt64Def := StrToInt64Def(ucParamStr(1, t), Trunc(ucParam(2, t)));
end;

function uc_StrToIntDef(var t: ucThreadType): Extended; stdcall;
begin
   uc_StrToIntDef := StrToIntDef(ucParamStr(1, t), Trunc(ucParam(2, t)));
end;

function uc_ChangeFileExt(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(ChangeFileExt(ucParamStr(1, t), ucParamStr(2, t)), t);
end;

function uc_ExpandFileName(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(ExpandFileName(ucParamStr(1, t)), t);
end;

function uc_ExpandUNCFileName(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(ExpandUNCFileName(ucParamStr(1, t)), t);
end;

function uc_ExtractFileDir(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(ExtractFileDir(ucParamStr(1, t)), t);
end;

function uc_ExtractFileDrive(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(ExtractFileDrive(ucParamStr(1, t)), t);
end;

function uc_ExtractFileExt(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(ExtractFileExt(ucParamStr(1, t)), t);
end;

function uc_ExtractFileName(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(ExtractFileName(ucParamStr(1, t)), t);
end;

function uc_ExtractFilePath(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(ExtractFilePath(ucParamStr(1, t)), t);
end;

function uc_ExtractRelativePath(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(ExtractRelativePath(ucParamStr(1, t), ucParamStr(2, t)), t);
end;

function uc_ExtractShortPathName(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(ExtractShortPathName(ucParamStr(1, t)), t);
end;

//function uc_MatchesMask(ucThread: Longint): Extended; stdcall;
//begin
    //uc_MatchesMask := MatchesMask(ucParamStr(1, ucThread), ucParamStr(2, ucThread));
//end;

function uc_DirectoryExists(var t: ucThreadType): Extended; stdcall;
begin
   uc_DirectoryExists := Byte(DirectoryExists(ucParamStr(1, t)));
end;

function uc_DiskFree(var t: ucThreadType): Extended; stdcall;
begin
   uc_DiskFree := DiskFree(Trunc(ucParam(1, t)));
end;

function uc_DiskSize(var t: ucThreadType): Extended; stdcall;
begin
   uc_DiskSize := DiskSize(Trunc(ucParam(1, t)));
end;

function uc_FileAge(var t: ucThreadType): Extended; stdcall;
begin
   uc_FileAge := FileAge(ucParamStr(1, t));
end;

function uc_FileDateToDateTime(var t: ucThreadType): Extended; stdcall;
begin
   uc_FileDateToDateTime := FileDateToDateTime(Trunc(ucParam(1, t)));
end;

function uc_FileExists(var t: ucThreadType): Extended; stdcall;
begin
   uc_FileExists := Byte(FileExists(ucParamStr(1, t)));
end;

function uc_FileGetAttr(var t: ucThreadType): Extended; stdcall;
begin
   uc_FileGetAttr := FileGetAttr(ucParamStr(1, t));
end;

function uc_FileSearch(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(FileSearch(ucParamStr(1, t), ucParamStr(2, t)), t);
end;

function uc_GetCurrentDir(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(GetCurrentDir, t);
end;

function uc_Chr(var t: ucThreadType): Extended; stdcall;
begin
   //+++
//    uc_Chr := Extended(Integer(Chr(Trunc(ucParam(1, t)))));
end;

function uc_CompareMem(var t: ucThreadType): Extended; stdcall;
begin
//+++ Boolean 1 or -1?
   uc_CompareMem := Integer(CompareMem(Pointer(Trunc(ucParam(1, t))),
                            Pointer(Trunc(ucParam(2, t))), Trunc(ucParam(3, t))));
end;

function uc_FormatMaskText(var t: ucThreadType): Extended; stdcall;
begin
   // uses Mask
   ucReturnStr(FormatMaskText(ucParamStr(1, t), ucParamStr(2, t)), t);
end;

function uc_Hi(var t: ucThreadType): Extended; stdcall;
begin
   uc_Hi := Hi(Trunc(ucParam(1, t)));
end;

function uc_IsValidIdent(var t: ucThreadType): Extended; stdcall;
begin
   // +++ Boolean 1 , -1
   uc_IsValidIdent := Integer(IsValidIdent(ucParamStr(1, t)));
end;

function uc_Lo(var t: ucThreadType): Extended; stdcall;
begin
   uc_Lo := Lo(Trunc(ucParam(1, t)));
end;

function uc_UpCase(var t: ucThreadType): Extended; stdcall;
begin
   ucReturnStr(UpCase(Char(ucParamStr(1, t))), t);
end;

function DelphiHex(var t: ucThreadType): Longint; stdcall;
begin
   if (ucElementCategory(t) = ucIsUnaryPreOperator) then
      if (ucTreeGetOp(t) = '$')
      then ucSetExpr('#H' + ucTreeGetParam(1, t), t);
end;

procedure ucDelphiLibInit(var t: ucThreadType); stdcall;
begin
   ucTreeParseUp(@uc_PreParse_StringArraySyntax);
   ucTreeParseDown(@DelphiHex);
   ucDefineOperator('$[x] = x');

   t.ucProperty.FunctionProperties := ucInline + ucOverload;

//  MATH
//  Arithmetic
   ucDefineFunction('Floor(x)', @uc_Floor);
   ucDefineFunction('Frac(x)', @uc_Frac);
   ucDefineFunction('Ldexp(X, P) = X * 2^P');
   ucDefineFunction('LnXP1(x)', @uc_LnXP1);
   ucDefineFunction('LogN(x, y)', @uc_LogN);
   ucDefineFunction('Pi()', @uc_Pi);
   ucDefineFunction('Poly(x, coefficients())', @uc_Poly);

   ucDefineFunction('Power(Base, Exponent) = Base ^ Exponent');
   ucDefineFunction('Round(x)', @uc_Round);
   ucDefineFunction('Trunc(x)', @uc_Trunc);

// Business
   ucDefineFunction('DoubleDecliningBalance(Cost, Salvage, Life, Period)', @uc_DoubleDecliningBalance);
   ucDefineFunction('FutureValue(Rate, NPeriods, Payment, PresentValue, PaymentTime)', @uc_FutureValue);
   ucDefineFunction('InterestPayment(Rate, Period, NPeriods, PresentValue, FutureValue, PaymentTime)', @uc_InterestPayment);

   ucDefineFunction('InterestRate(NPeriods, Payment, PresentValue, FutureValue, PaymentTime)', @uc_InterestRate);
   ucDefineFunction('InternalRateOfReturn(Guess, CashFlows())', @uc_InternalRateOfReturn);
   ucDefineFunction('NetPresentValue(Rate, CashFlows(), PaymentTime)', @uc_NetPresentValue);
   ucDefineFunction('NumberOfPeriods(Rate, Payment, PresentValue, FutureValue, PaymentTime)', @uc_NumberOfPeriods);
   ucDefineFunction('Payment(Rate, NPeriods, PresentValue, FutureValue, PaymentTime)', @uc_Payment);
   ucDefineFunction('PeriodPayment(Rate, Period, NPeriods, PresentValue, FutureValue, PaymentTime)', @uc_PeriodPayment);
   ucDefineFunction('PresentValue(Rate, NPeriods, Payment, FutureValue, PaymentTime)', @uc_PresentValue);
   ucDefineFunction('SLNDepreciation(Cost, Salvage, Life)', @uc_SLNDepreciation);
   ucDefineFunction('SYDDepreciation(Cost, Salvage, Life, Period)', @uc_SYDDepreciation);

// Measurement conversion routines
   ucDefineFunction('CycleToRad(Cycles) = 2*Pi*Cycles');
   ucDefineFunction('DegToRad(Degrees) = Degrees * Pi/180');
   ucDefineFunction('GradToRad(Grads) = Grads * Pi/200');
   ucDefineFunction('RadToCycle(Radians) = Radians / (2*Pi)');
   ucDefineFunction('RadToDeg(Radians) = Radians * 180/Pi');
   ucDefineFunction('RadToGrad(Radians) = Radians(200/Pi)');

// Ordinal Routines
//   ucDefineFunction('Dec', @uc_); +++
//   ucDefineFunction('Inc', @uc_); +++
   ucDefineFunction('Odd(X) = Abs(X MOD 2)');
//   ucDefineFunction('Ord', @uc_); +++
//   ucDefineFunction('Pred', @uc_); +++
//   ucDefineFunction('Succ', @uc_); +++

// Random number routines
   ucDefineFunction('RandG(Mean, StdDev)', @uc_RandG);
   ucDefineFunction('Random(Range=1)', @uc_Random);

// Statistical routines
   ucDefineFunction('MaxIntValue(Data())', @uc_MaxIntValue);
   ucDefineFunction('MaxValue(Data())', @uc_MaxValue);
   ucDefineFunction('Mean(Data())', @uc_Mean);
//+++   ucDefineFunction('MeanAndStdDev(Data: array of Double; var Mean, StdDev: Extended)', @uc_);
   ucDefineFunction('MinIntValue(Data())', @uc_MinIntValue);
   ucDefineFunction('MinValue(Data())', @uc_MinValue);
//+++   ucDefineFunction('MomentSkewKurtosis(const Data: array of Double; var M1, M2, M3, M4, Skew, Kurtosis: Extended)', @uc_);
   ucDefineFunction('Norm(Data())', @uc_Norm);
   ucDefineFunction('PopnStdDev(Data())', @uc_PopnStdDev);
   ucDefineFunction('PopnVariance(Data())', @uc_PopnVariance);
   ucDefineFunction('StdDev(Data())', @uc_StdDev);
   ucDefineFunction('Sum(Data())', @uc_Sum);
   ucDefineFunction('SumInt(Data())', @uc_SumInt);
   ucDefineFunction('SumOfSquares(Data())', @uc_SumOfSquares);
//+++   ucDefineFunction('SumsAndSquares(const Data: array of Double; var Sum, SumOfSquares: Extended)', @uc_);
   ucDefineFunction('TotalVariance(Data())', @uc_TotalVariance);
   ucDefineFunction('Variance(Data())', @uc_Variance);

// Trigonometry
   ucDefineFunction('ArcCos(x) = ACOS(x)');
   ucDefineFunction('ArcCosh(x) = ACOSH(x)');
   ucDefineFunction('ArcSin(x) = ASIN(x)');
   ucDefineFunction('ArcSinh(x) = ASINH(x)');
   ucDefineFunction('ArcTan(x) = ATAN(x)');
   ucDefineFunction('ArcTan2(x, y)', @uc_ArcTan2);
   ucDefineFunction('ArcTanh(x) = ATANH(x)');
   ucDefineFunction('Cotan(x) = COT(x)');
   ucDefineFunction('Hypot(x, y) = Sqrt(x^2 + y^2)');
//+++   ucDefineFunction('SinCos(Theta: Extended; var Sin, Cos: Extended)', @uc_);

// Strings, Type conversions, formatting
// Date/Time routines
   ucDefineFunction('Date()', @uc_Date);
   ucDefineFunction('DateTimeToFileDate(DateTime)', @uc_DateTimeToFileDate);
   ucDefineFunction('DateTimeToStr$(DateTime)', @uc_DateTimeToStr);
//+++   ucDefineFunction('DateTimeToString(var NewString: string; const Format: string; DateTime: TDateTime);', @uc_);
//      ucDefineFunction('procedure DateTimeToSystemTime(DateTime: TDateTime; var SystemTime: TSystemTime);', @uc_);
//+++   ucDefineFunction('DateTimeToTimeStamp(DateTime)', @uc_DateTimeToTimeStamp);
   ucDefineFunction('DateToStr$(Date)', @uc_DateToStr);
   ucDefineFunction('DayOfWeek(Date)', @uc_DayOfWeek);
//+++   ucDefineFunction('DecodeDate(Date: TDateTime; var Year, Month, Day: Word);', @uc_);
//+++   ucDefineFunction('DecodeTime(Time: TDateTime; var Hour, Min, Sec, MSec: Word);', @uc_);
   ucDefineFunction('EncodeDate(Year, Month, Day)', @uc_EncodeDate);
   ucDefineFunction('EncodeTime(Hour, Min, Sec, MSec)', @uc_EncodeTime);
   ucDefineFunction('FormatDateTime(Format$, DateTime)', @uc_FormatDateTime);
   ucDefineFunction('IncMonth(Date, NumberOfMonths)', @uc_IncMonth);
   ucDefineFunction('IsLeapYear(Year)', @uc_IsLeapYear);
//+++   ucDefineFunction('MSecsToTimeStamp(MSecs)', @uc_MSecsToTimeStamp);
   ucDefineFunction('Now()', @uc_Now);
   ucDefineFunction('StrToDate(S$)', @uc_StrToDate);
   ucDefineFunction('StrToDateTime(S$)', @uc_StrToDateTime);
   ucDefineFunction('StrToTime(S$)', @uc_StrToTime);
//+++   ucDefineFunction('SystemTimeToDateTime(SystemTime)', @uc_SystemTimeToDateTime);
   ucDefineFunction('Time()', @uc_Time);
//+++   ucDefineFunction('TimeStampToDateTime(TimeStamp)', @uc_TimeStampToDateTime);
//+++   ucDefineFunction('TimeStampToMSecs(TimeStamp)', @uc_TimeStampToMSecs);
   ucDefineFunction('TimeToStr$(Time)', @uc_TimeToStr);

// Floating point conversion routines
   ucDefineFunction('FloatToStr$(Value)', @uc_FloatToStr);
   ucDefineFunction('FloatToStrF$(Value, Format, Precision, Digits)', @uc_FloatToStrF);
//+++   ucDefineFunction('FloatToText(Buffer: PChar; const Value; ValueType: TFloatValue; Format: TFloatFormat; Precision, Digits: Integer)', @uc_);
//+++   ucDefineFunction('FloatToTextFmt(Buffer: PChar; const Value; ValueType: TFloatValue; Format: PChar): Integer;', @uc_);
   ucDefineFunction('FormatFloat$(Format$, Value)', @uc_FormatFloat);
   ucDefineFunction('StrToCurr(S$)', @uc_StrToCurr);
   ucDefineFunction('StrToFloat(S$)', @uc_StrToFloat);
//+++   ucDefineFunction('TextToFloat(Buffer: PChar; var Value; ValueType: TFloatValue)', @uc_);

// MBCS conversion routines
   ucDefineFunction('AnsiCompareFileName(S1$, S2$)', @uc_AnsiCompareFileName);
//+++   ucDefineFunction('AnsiLastChar(const S: string): PChar;', @uc_);
   ucDefineFunction('AnsiLowerCaseFileName$(S$)', @uc_AnsiLowerCaseFileName);
//+++   ucDefineFunction('AnsiStrLastChar(P: PChar): PChar', @uc_);
   ucDefineFunction('AnsiUpperCaseFileName$(S$)', @uc_AnsiUpperCaseFileName);
   ucDefineFunction('ByteToCharIndex(S$, Index)', @uc_ByteToCharIndex);
   ucDefineFunction('ByteToCharLen(S$, MaxLen)', @uc_ByteToCharLen);
//+++   ucDefineFunction('ByteType(S$, Index)', @uc_ByteType);
   ucDefineFunction('CharToByteIndex(S$, Index)', @uc_CharToByteIndex);
   ucDefineFunction('CharToByteLen(S$, MaxLenr)', @uc_CharToByteLen);
//+++   ucDefineFunction('StrByteType(Str: PChar; Index: Cardinal)', @uc_StrByteType);

// Numeric formatting routines
   ucDefineFunction('CurrToStr$(Value)', @uc_CurrToStr);
   ucDefineFunction('CurrToStrF$(Value, Format, Digits)', @uc_CurrToStrF);
   ucDefineFunction('FormatCurr$(Format$, Value)', @uc_FormatCurr);
   ucDefineFunction('IntToHex$(Value, Digits)', @uc_IntToHex);
   ucDefineFunction('IntToStr$(Value)', @uc_IntToStr);

// String formatting routines
//+++   ucDefineFunction('FmtLoadStr(Ident: Integer; const Args: array of const): string;', @uc_);

// String handling routines
   ucDefineFunction('AdjustLineBreaks$(S$)', @uc_AdjustLineBreaks);
   ucDefineFunction('AnsiCompareStr(S1$, S2$)', @uc_AnsiCompareStr);
   ucDefineFunction('AnsiCompareText(S1$, S2$)', @uc_AnsiCompareText);
//+++   ucDefineFunction('AnsiExtractQuotedStr$(Src$, Quote$)g', @uc_AnsiExtractQuotedStr);
//   overload
   ucDefineFunction('AnsiLowerCase$(S$)', @uc_AnsiLowerCase);
   ucDefineFunction('AnsiPos(Substr$, S$);', @uc_AnsiPos);
   ucDefineFunction('AnsiQuotedStr$(S$, Quote$)', @uc_AnsiQuotedStr);
   ucDefineFunction('AnsiUpperCase$(S$)', @uc_AnsiUpperCase);
   ucDefineFunction('CompareStr(S1$, S2$);', @uc_CompareStr);
   ucDefineFunction('CompareText(S1$, S2$)', @uc_CompareText);
//+++   ucDefineFunction('Concat(s1 [, s2,..., sn]: string): string;', @uc_);
   ucDefineFunction('Copy$(S$, Index, Count)', @uc_Copy);
//+++   ucDefineFunction('Insert(Source: string; var S: string; Index: Integer);', @uc_);
   ucDefineFunction('IsDelimiter(Delimiters$, S$, Index)', @uc_IsDelimiter);
   ucDefineFunction('IsPathDelimiter(S$, Index)', @uc_IsPathDelimiter);
   ucDefineFunction('LastDelimiter(Delimiters$, S$)', @uc_LastDelimiter);
//   ucDefineFunction('Length(S$)', @uc_Length);  conflicts with area(length,width) in DemoDlph
   ucDefineFunction('LowerCase$(S$)', @uc_LowerCase);
   ucDefineFunction('Pos(Substr$, S$)', @uc_Pos);
   ucDefineFunction('QuotedStr$(S$)', @uc_QuotedStr);
   ucDefineFunction('StringOfChar$(Ch$, Count)', @uc_StringOfChar);
//   overload
//+++   ucDefineFunction('StringReplace$(S$, OldPattern$, NewPattern$, Flags)', @uc_StringReplace);
   ucDefineFunction('Trim$(S$)', @uc_Trim);
   ucDefineFunction('TrimLeft$(S$)', @uc_TrimLeft);
   ucDefineFunction('TrimRight$(S$)', @uc_TrimRight);
   ucDefineFunction('UpperCase$(S$)', @uc_UpperCase);
//+++   ucDefineFunction('WrapText(const Line, BreakStr: string; nBreakChars: TSysCharSet; MaxCol: Integer):string', @uc_);

// String handling routines (null-terminated)
   ucDefineFunction('AnsiStrComp(S1$, S2$)', @uc_AnsiStrComp);
   ucDefineFunction('AnsiStrIComp(S1$, S2$)', @uc_AnsiStrIComp);
   ucDefineFunction('AnsiStrLComp(S1$, S2$, MaxLen)', @uc_AnsiStrLComp);
   ucDefineFunction('AnsiStrLIComp(S1$, S2$, MaxLen)', @uc_AnsiStrLIComp);
   ucDefineFunction('AnsiStrLower$(Str$)', @uc_AnsiStrLower);
   ucDefineFunction('AnsiStrPos$(Str$, SubStr$)', @uc_AnsiStrPos);
//+++   ucDefineFunction('AnsiStrRScan(Str: PChar; Chr: Char): PChar;', @uc_);
   ucDefineFunction('AnsiStrScan$(Str$, Chr$)', @uc_AnsiStrScan);
   ucDefineFunction('AnsiStrUpper$(Str$)', @uc_AnsiStrUpper);
   ucDefineFunction('LineStart$(Buffer$, BufPos$)', @uc_LineStart);
//+++   ucDefineFunction('StrAlloc(Size: Cardinal): PChar;', @uc_);
//+++   ucDefineFunction('StrBufSize(Str: PChar): Cardinal;', @uc_);
   ucDefineFunction('StrCat$(Dest$, Source$)', @uc_StrCat);
   ucDefineFunction('StrComp(Str1$, Str2$)', @uc_StrComp);
   ucDefineFunction('StrCopy$(Dest$, Source$)', @uc_StrCopy); //+++
//+++   ucDefineFunction('StrEnd(Str: PChar): PChar;', @uc_);
//+++   ucDefineFunction('StrIComp(Str1$, Str2$): Integer', @uc_StrIComp);
   ucDefineFunction('StrLCat$(Dest$, Source$, MaxLen)', @uc_StrLCat);
   ucDefineFunction('StrLComp(Str1$, Str2$, MaxLen)', @uc_StrLComp);
   ucDefineFunction('StrLCopy$(Dest$, Source$, MaxLen)', @uc_StrLCopy);
   ucDefineFunction('StrLen(Str$)', @uc_StrLen);
   ucDefineFunction('StrLIComp(Str1$, Str2$, MaxLen)', @uc_StrLIComp);
   ucDefineFunction('StrLower$(Str$)', @uc_StrLower);
   ucDefineFunction('StrMove$(Dest$, Source$, Count)', @uc_StrMove);
//+++   ucDefineFunction('StrNew(Str: PChar): PChar;', @uc_);
   ucDefineFunction('StrPCopy$(Dest$, Source$)', @uc_StrPCopy);
   ucDefineFunction('StrPLCopy$(Dest$, Source$, MaxLen)', @uc_StrPLCopy);
   ucDefineFunction('StrPos$(Str1$, Str2$)', @uc_StrPos);
   ucDefineFunction('StrRScan$(Str$, Chr$)', @uc_StrRScan);
   ucDefineFunction('StrScan$(Str$, Chr$)', @uc_StrScan);
   ucDefineFunction('StrUpper$(Str$)', @uc_StrUpper);

// Type conversion routines
   ucDefineFunction('StrToInt(S$)', @uc_StrToInt);
   ucDefineFunction('StrToInt64(S$)', @uc_StrToInt64);
   ucDefineFunction('StrToInt64Def(S$, Default)', @uc_StrToInt64Def);
   ucDefineFunction('StrToIntDef(S$, Default)', @uc_StrToIntDef);

// FILE UTILITIES
// File name utilities
   ucDefineFunction('ChangeFileExt$(FileName$, Extention$)', @uc_ChangeFileExt);
   ucDefineFunction('ExpandFileName$(FileName$)', @uc_ExpandFileName);
   ucDefineFunction('ExpandUNCFileName$(FileName$)', @uc_ExpandUNCFileName);
   ucDefineFunction('ExtractFileDir$(FileName$)', @uc_ExtractFileDir);
   ucDefineFunction('ExtractFileDrive$(FileName$)', @uc_ExtractFileDrive);
   ucDefineFunction('ExtractFileExt$(FileName$)', @uc_ExtractFileExt);
   ucDefineFunction('ExtractFileName$(FileName$)', @uc_ExtractFileName);
   ucDefineFunction('ExtractFilePath$(FileName$)', @uc_ExtractFilePath);
   ucDefineFunction('ExtractRelativePath$(BaseName$, DestName$)', @uc_ExtractRelativePath);
   ucDefineFunction('ExtractShortPathName$(FileName$)', @uc_ExtractShortPathName);
//   ucDefineFunction('MatchesMask(FileName$, Mask$)', @MatchesMask);

// File management routines
   ucDefineFunction('DirectoryExists(Name$)', @uc_DirectoryExists);
   ucDefineFunction('DiskFree(Drive)', @uc_DiskFree);
   ucDefineFunction('DiskSize(Byte)', @uc_DiskSize);
   ucDefineFunction('FileAge(FileName$)', @uc_FileAge);
   ucDefineFunction('FileDateToDateTime(FileDate)', @uc_FileDateToDateTime);
   ucDefineFunction('FileExists(FileName$)', @uc_FileExists);
   ucDefineFunction('FileGetAttr(FileName$)', @uc_FileGetAttr);
   ucDefineFunction('FileSearch$(Name$, DirList$)', @uc_FileSearch);
   ucDefineFunction('GetCurrentDir$()', @uc_GetCurrentDir);

// Misc
//+++   ucDefineFunction('Chr(X: Byte): Char;', @uc_);
   ucDefineFunction('CompareMem(Pointer1, Pointer2, Length)', @uc_CompareMem);
   ucDefineFunction('FormatMaskText$(EditMask$, Value$)', @uc_FormatMaskText);
   ucDefineFunction('Hi(X)', @uc_Hi);
//+++   ucDefineFunction('High(X)', @uc_);
   ucDefineFunction('IsValidIdent(Ident$)', @uc_IsValidIdent);
   ucDefineFunction('Lo(X)', @uc_Lo);
//+++   ucDefineFunction('LoadStr(Ident: Integer): string;', @uc_);
//+++   ucDefineFunction('Low(X);', @uc_);
//+++   ucDefineFunction('SizeOf(X): Integer;', @uc_);
   ucDefineFunction('UpCase$(Ch$)', @uc_UpCase);
end;

procedure ucLibInit(var t: ucThreadType); stdcall;
begin
   ucDelphiLibInit(t);
end;

exports

   ucDelphiLibInit, ucLibInit;

begin
end.
