// ucalc.h : Header file
// Designed to work with 32-bit MS Visual C++ and Borland C++ Builder.
//
// UCalc Fast Math Parser (Beta J)
// Copyright (c) 1998-2004 by Daniel Corbier.  All rights reserved.
// http://www.ucalc.com/mathparser

#ifndef __UCALC__
#define __UCALC__

#include <wtypes.h>
#include <string.h>

struct ucDefinitionSpaceType {
   long BaseDefinitionSpace;
   long DefinitionSpace;
   long InlineBuffer;
   long MaxErrorData;
   long MaxThreadCount;
};

struct ucForInternalUseType {
   long  LanguageID;
   DWORD ucOperator;
   DWORD ucEvalAddr;
   DWORD ucEvaluateAddr;
   DWORD ucThreadAddr;
   DWORD ParamListAddr;
   DWORD HandleAddr;
   DWORD ReleaseExprPtr;
   long TreeIndex;
   long TreeParamInfoAddress;
   long HaltEvaluation;
   long ThreadStatus;
   long PrevSCount;
   long PrevLCount;
   long CompareOld;
   long IsStringValue;
   long ucStack;
   long Infinity;
   long double *TrigFactor;
   long double *TrigFactorInv;
   long im;
};

struct ucPropertyType {
   long DefaultNumericType;
   long DefaultStringType;
   long FPU_ControlWord_UCalc;
   long FPU_ControlWord_HostProgram;
   long CaseSensitive;
   long ComplexNumbers;
   long ImagSymbol;
   long RelationalTrue;
   long ImplicitMultiply;
   long MaxParamPerFunction;
   long ArrayProperties;
   long FunctionProperties;
   long StackSize;
   long DefaultPrecedence;
   long RandomSeed;
   long OutputDigits;
   long QuoteChar;
   long QuoteCharSynonym;
   long ArgumentSeparator;
   long DecimalSeparator;
   long StatementSeparator;
   ucDefinitionSpaceType DefSpace;
};

struct ucHandleType {
   long ucThread;
   long Info;
   long DataType;
   DWORD DataPtr;
   DWORD DataCplxPtr;
   DWORD SymbolName;
   DWORD SortedHandle;
   DWORD OverloadHandle;
   DWORD OpPrecedence;
   DWORD ParamCount;
   DWORD CallbackPtr;
   DWORD FuncArgs;
   DWORD FuncArgHandleData;
   DWORD VarValueStr;

   DWORD UserData1;
   DWORD UserData2;
   DWORD UserData3;
   DWORD UserData4;
};

struct ucThreadType {
   long ucThread;
   long Context;
   long ParamCount;
   long ErrorNumber;
   long TreeDirection;
   long DefSpace;
   long scount;
   long lcount;
   long Sequence;
   long SequenceLast;
   long ExprHasChanged;
   long IsFunctionParse;
   long ComplexStatus;
   DWORD ucParamAddr;
   DWORD ucParamHandleAddr;
   long SymbolStart;
   long SymbolEnd;
   long PrecedenceMax;
   long PrecedenceMin;
   long VarHandle;
   long OpHandle;
   DWORD ReturnString;
   long double *ReturnValue;
   long double *ReturnComplex;
   ucPropertyType ucProperty;
   ucForInternalUseType Handshake;
   long User1;
   long User2;
   long User3;
   long User4;
};

struct ucComplexNumberType {
   long double Real;
   long double Imag;
};

const ucTrue  = 1;
const ucFalse = 0;

// LanguageID
const uc_Lang_Unsupported    = 0;
const uc_Lang_Debug          = 1;
const uc_Lang_PowerBasic     = 2;
const uc_Lang_Delphi         = 3;
const uc_Lang_VisualBasic    = 4;
const uc_Lang_VisualBasicNET = 5;
const uc_Lang_CPlusPlus      = 6;

// Type constants for vType arg of ucDefineFunction and ucDefineVariable
const ucType_None          =-2;
const ucType_Constant      =-1;
const ucType_Internal      = 0;
const ucType_Integer8      = 1;
const ucType_Integer8u     = 2;
const ucType_Integer16     = 3;
const ucType_Integer16u    = 4;
const ucType_Integer32     = 5;
const ucType_Integer32u    = 6;
const ucType_Integer64     = 7;
const ucType_Integer64u    = 8;
const ucType_Currency      = 9;
const ucType_CurrencyExt   = 10;
const ucType_FloatSingle   = 11;
const ucType_FloatDouble   = 12;
const ucType_FloatExtended = 13;
const ucType_StringBSTR    = 14;  // For VB
const ucType_StringLPSTR   = 15;  // (ASCIIZ) C++ char default
const ucType_StringAnsi    = 16;  // PowerBASIC and Delphi default

const ucType_Long          = ucType_Integer32;
const ucType_Double        = ucType_FloatDouble;
const ucType_Extended      = ucType_FloatExtended;
const ucType_AnsiString    = ucType_StringAnsi;

// Context options (t.Context)
const ucContext_ucEval            = 1;
const ucContext_ucEvalStr         = 2;
const ucContext_ucEvaluateStr     = 4;
const ucContext_ucParse           = 8;
const ucContext_ucParseOrucEval   = 16;
const ucContext_ucDefineFunction  = 32;
const ucContext_ucDefineVariable  = 64;
const ucContext_ucReset           = 128;
const ucContext_MultiLineDef      = 256;
const ucContext_MultiLineDefFirst = 512;
const ucContext_MultiLineDefLast  = 1024;

// For ucTreeProperty, ucTreeNodeProperty,
// ucTreeParamProperty, and ucTreeParentProperty    
const ucIsInline          = 0;
const ucIsDynamic         = 1;
const ucIsFastCallback    = 2;
const ucIsLocal           = 3;
const ucIsComplex         = 4;
const ucIsLocked          = 5;
const ucIsConstant        = ucIsLocked;
const ucIsOverride        = 6;
const ucIsOverwrite       = 7;
const ucIsOverload        = 8;
const ucIsRecursion       = 9;
const ucIsBootstrap       = 10;
const ucIsMultiLine       = 11;
const ucIsEndMultiLine    = ucIsMultiLine;

const ucIsArray           = 12;

const ucIsFunction        = 13;
const ucIsOperator        = 14;
const ucIsBinaryOperator    = 15;
const ucIsUnaryPostOperator = 16;
const ucIsUnaryPreOperator  = 17;
const ucIsVariable        = 18;
const ucIsJITVariable     = 19;
const ucIsJITExpression   = 20;
const ucIsString          = 21;
const ucIsInternal        = 22;
const ucIsExternal        = 23;
const ucIsActive          = 24;
const ucIsLCount          = 25;
const ucIsSorted          = 26;
const ucIsMaybeComplex    = 27;
const ucHasComplexAltDef  = 28;
const PassAnArray         = 29;
const Reserved1           = 30;
const Reserved2           = 31;

const ucBinaryOp  = ucIsBinaryOperator;
const ucUnaryPost = ucIsUnaryPostOperator;
const ucUnaryPre  = ucIsUnaryPreOperator;

// For ucElementCategory
const ucIsOperand           = 32;
// ucIsFunction          = (already defined)
const ucIsFunctionArg       = 33;
// ucIsBinaryOperator    = (already defined)
// ucIsUnaryPreOperator  = (already defined)
// ucIsUnaryPostOperator = (already defined)

const ucDataType    = 40;
const ucResultPtr   = 41;
const ucSignPos     = 42;
const ucElementCat  = 43;
const ucTreeIndex   = 44;
const ucErrorNumber = 45;
const ucOpHandle    = 46;
const ucFuncHandle  = ucOpHandle;
const ucExprPtr     = 47;
const ucCodeAddress = 48;
const ucCurrentPtr  = 49;
const ucParentPtr   = 50;
const ucDataHandle  = 51;
const ucMiscData    = 52;
const ucUserData1   = 53;
const ucUserData2   = 54;
const ucUserData3   = 55;
const ucUserData4   = 56;

const ucTreePre  = 5;
const ucTreeDownRaw = 6;
const ucTreeDown = 7;
const ucTreeUp   = 8;

const ucErrorCallback = 9;

const ucNoChange = 0;
const ucMinorChange = 1;
const ucMajorChange = 2;

// Trig mode options for ucSetTrigMode
const ucRadians = 1;
const ucDegrees = 2;
const ucGrads   = 3;
const ucCycles  = 4;

// Options for ucArrayInfo,
// and the first two for ucProperties.ArrayProperties
const ucArray_BoundsCheck    = 1;
const ucArray_ColumnMajor    = 2;
const ucArray_DataAddress    = 3;
const ucArray_DataType       = 4;
const ucArray_DimensionCount = 5;
const ucArray_ElementCount   = 6;
const ucArray_ElementSize    = 7;

// Options for fProperties parameter of ucDefineFunction
// and for ucProperties.FunctionProperties
const ucInline       = 1;
const ucDynamic      = 2;
const ucFastCallback = 4;
const ucLocal        = 8;
const ucComplex      = 16;
const ucLocked       = 32;
const ucConstant     = ucLocked;
const ucOverride     = 64;
const ucOverwrite    = 128;
const ucOverload     = 256;
const ucRecursion    = 512;
const ucBootstrap    = 1024;
const ucMultiLine    = 2048;
const ucEndMultiLine = ucMultiLine;

// FPU control word values for ucProperties.FPU_ControlWord,
// and SetFPU(), SetOrFPU, SetXorFPU, GetFPU(), RestoreFPU
const ucFPU_ErrMask_InvalidOp      = 1;
const ucFPU_ErrMask_DenormalOp     = 2;
const ucFPU_ErrMask_ZeroDivide     = 4;
const ucFPU_ErrMask_Overflow       = 8;
const ucFPU_ErrMask_Underflow      = 16;
const ucFPU_ErrMask_PrecisionLoss  = 32;

const ucFPU_Precision_Single   = 0;
const ucFPU_Precision_Double   = 256;
const ucFPU_Precision_Extended = 768;

const ucFPU_Round_Even   = 0;
const ucFPU_Round_Down   = 1024;
const ucFPU_Round_Up     = 2048;
const ucFPU_Round_Toward = 3072;

// For ucSetErrorData / ucGetErrorData
const ucNewIndex = -2;
const ucNewUniqueIndex  = -1;
const ucAllErrorData    = -1;

// Error Values
const ucErr_None                = 0;
const ucErr_ucErrorData         = 1;
const ucErr_SyntaxError         = 2;
const ucErr_MismatchedParenth   = 3;
const ucErr_MismatchedQuotes    = 4;
const ucErr_UndefinedVariable   = 5;
const ucErr_UndefinedFunction   = 6;
const ucErr_IncorrectArgType    = 7;
const ucErr_InvalidNumberOfArgs = 8;
const ucErr_InvalidOperandType  = 9;
const ucErr_InvalidName         = 10;
const ucErr_ImplicitMult        = 11;

const ucErr_InvalidOperation    = 20;
const ucErr_DenormalOperand     = 21;
const ucErr_DivisionByZero      = 22;
const ucErr_Overflow            = 23;
const ucErr_Underflow           = 24;
const ucErr_InexactResult       = 25;
const ucErr_SubscriptOutOfRange = 35;
const ucErr_StackOverflow       = 36;

const ucErr_OperationDenied     = 40;
const ucErr_PropertyConflict    = 41;
const ucErr_ConstantWriteDenied = 42;
const ucErr_FunctionDefError    = 43;
const ucErr_FunctionLocked      = 44;

const ucErr_DefinitionSpaceFull = 60;
const ucErr_ThreadInitFailed    = 61;
const ucErr_UnsupportedFeature  = 62;


#if defined(__BORLANDC__)
const ucType = ucType_Extended;
typedef long double ucNUM;
#else
const ucType = ucType_Extended; // +++ucType_Double;
typedef double ucNUM;
#endif

typedef long  (CALLBACK* UCALIASE)(const char*, const char*, long);
typedef DWORD (CALLBACK* UCARRAYINFOE)(long, long);
typedef long  (CALLBACK* UCDEFINEFUNCTIONE)(const char*, DWORD, long, long, long, long);
typedef long  (CALLBACK* UCDEFINEVARIABLEE)(const char*, DWORD, DWORD, long, long, long);
typedef long double (CALLBACK* UCEVALE)(const char*, long);
typedef char* (CALLBACK* UCEVALSTRE)(const char*, long);
typedef long double (CALLBACK* UCEVALUATEE)(DWORD);
typedef char* (CALLBACK* UCEVALUATESTRE)(DWORD);
typedef char* (CALLBACK* UCGETERRORDATAE)(long, long);
typedef char* (CALLBACK* UCGETPARAMSTRE)(long, ucThreadType&);
typedef char* (CALLBACK* UCTREEGETPARAME)(long, ucThreadType&);
typedef long  (CALLBACK* UCTREEPROPERTYE)(DWORD, long, long, ucThreadType&);
typedef DWORD (CALLBACK* UCGETTHREADINFOE)(long, ucThreadType&);
typedef char* (CALLBACK* UCGETVARIABLEVALUESTRE)(long);
typedef long  (CALLBACK* UCLOWERBOUNDE)(long, long);
typedef long  (CALLBACK* UCNEWTHREADE)(void);
typedef long  (CALLBACK* UCNEWSYMBOLHANDLEE)(ucThreadType&);
typedef long  (CALLBACK* UCPARAMHANDLEE)(long, ucThreadType&);
typedef long  (CALLBACK* UCPARAMTYPEE)(long, ucThreadType&);
typedef DWORD (CALLBACK* UCPARSEE)(const char*, const char*, long, long);
typedef long  (CALLBACK* UCPRECEDENCEE)(const char*, long, long);
typedef long  (CALLBACK* UCSYMBOLHANDLEE)(const char*, long, long);
typedef long  (CALLBACK* UCTHREADADDRE)(long);
typedef long  (CALLBACK* UCUPPERBOUNDE)(long, long);

typedef void  (CALLBACK* UCDETACHSYMBOLE)(const char*, long);
typedef void  (CALLBACK* UCERRORCLEARE)(long);
typedef void  (CALLBACK* UCEVALUATEINITE)(void);
typedef void  (CALLBACK* UCLICENSEE)(const char*);
typedef long  (CALLBACK* UCSETCALLBACKE)(DWORD, long, long);
typedef void  (CALLBACK* UCRELEASEEXPRPTRE)(DWORD);
typedef void  (CALLBACK* UCRELEASEITEME)(long);
typedef void  (CALLBACK* UCRELEASETHREADE)(long);
typedef void  (CALLBACK* UCRESETE)(ucThreadType&);
typedef void  (CALLBACK* UCRETURNE)(long double, ucThreadType&);
typedef void  (CALLBACK* UCRETURNCPLXE)(long double, ucThreadType&);
typedef void  (CALLBACK* UCRETURNSTRE)(const char*, ucThreadType&);
typedef void  (CALLBACK* UCSETERRORDATAE)(long, const char*, ucThreadType&);
typedef void  (CALLBACK* UCSETPARAMSTRE)(long, const char*, ucThreadType&);
typedef void  (CALLBACK* UCSETTHREADADDRE)(long, ucThreadType&);
typedef void  (CALLBACK* UCSETTRIGMODEE)(long, long);
typedef void  (CALLBACK* UCSETVARIABLEVALUEE)(long, long double);
typedef void  (CALLBACK* UCSETVARIABLEVALUESTRE)(long, const char*);
typedef void  (CALLBACK* UCTREESETPARAME)(long, const char*, ucThreadType&);

typedef long double (CALLBACK* UCGETPARAME)(long, ucThreadType&);
typedef long double (CALLBACK* UCGETPARAMCPLXE)(long, ucThreadType&);
typedef long double (CALLBACK* UCGETVARIABLEVALUEE)(long);
typedef long double (CALLBACK* UCIMAGPARTE)(long);

extern void  ucReset(ucThreadType& t);
extern char* ucGetErrorMessage(long ErrorNumber, long ucThread);
extern DWORD ucGetThreadInfo(long ucThread, ucThreadType& t);
extern void  ucSetThreadAddr(long ucThread, ucThreadType& t);
extern DWORD ucThreadAddr(long ucThread);

extern long ucInitialized;
extern long ucLoadLibFailed;
extern ucThreadType t0;
extern ucHandleType *ucItem;

extern UCARRAYINFOE ucArrayInfoE;
extern UCALIASE ucAliasE;
extern UCDETACHSYMBOLE ucDetachSymbolE;
extern UCDEFINEFUNCTIONE ucDefineFunctionE;
extern UCDEFINEVARIABLEE ucDefineVariableE;
extern UCERRORCLEARE ucErrorClearE;
extern UCEVALE ucEvalE;
extern UCEVALSTRE ucEvalStrE;
extern UCEVALUATEE ucEvaluateE;
extern UCEVALUATEINITE ucEvaluateInitE;
extern UCEVALUATESTRE ucEvaluateStrE;
extern UCGETERRORDATAE ucGetErrorDataE;
extern UCGETPARAME ucGetParamE;
extern UCGETPARAMCPLXE ucGetParamCplxE;
extern UCGETPARAMSTRE ucGetParamStrE;
extern UCTHREADADDRE ucThreadAddrE;
extern UCTREEGETPARAME ucTreeGetParamE;
extern UCTREEPROPERTYE ucTreePropertyE;
extern UCGETTHREADINFOE ucGetThreadInfoE;
extern UCGETVARIABLEVALUEE ucGetVariableValueE;
extern UCGETVARIABLEVALUESTRE ucGetVariableValueStrE;
extern UCIMAGPARTE ucImagPartE;
extern UCLICENSEE ucLicenseE;
extern UCLOWERBOUNDE ucLowerBoundE;
extern UCNEWTHREADE ucNewThreadE;
extern UCPARAMHANDLEE ucParamHandleE;
extern UCPARAMTYPEE ucParamTypeE;
extern UCNEWSYMBOLHANDLEE ucNewSymbolHandleE;
extern UCPARSEE ucParseE;
extern UCPRECEDENCEE ucPrecedenceE;
extern UCSETCALLBACKE ucSetCallbackE;
extern UCRELEASEEXPRPTRE ucReleaseExprPtrE;
extern UCRELEASEITEME ucReleaseItemE;
extern UCRELEASETHREADE ucReleaseThreadE;
extern UCRESETE ucResetE;
extern UCRETURNE ucReturnE;
extern UCRETURNCPLXE ucReturnCplxE;
extern UCRETURNSTRE ucReturnStrE;
extern UCSETERRORDATAE ucSetErrorDataE;
extern UCSETPARAMSTRE ucSetParamStrE;
extern UCTREESETPARAME ucTreeSetParamE;
extern UCSETTHREADADDRE ucSetThreadAddrE;
extern UCSETTRIGMODEE ucSetTrigModeE;
extern UCSETVARIABLEVALUEE ucSetVariableValueE;
extern UCSETVARIABLEVALUESTRE ucSetVariableValueStrE;
extern UCSYMBOLHANDLEE ucSymbolHandleE;
extern UCTHREADADDRE ucThreadAddrE;
extern UCUPPERBOUNDE ucUpperBoundE;

class FastMath
{
public:
   FastMath();
   ~FastMath();

   long  ucDefineConstant(const char* ConstantDef, long vProperties);
   long  ucDefineFunction(const char* FunctionDef, void *FunctionAddress, long fProperties, long vType);
   long  ucDefineOperator(const char* OperatorDef, long Precedence, void *OperatorAddress, long fProperties, long vType);
   long  ucDefineVariable(const char* VariableDef, long vProperties);
   long  ucAttachVariable(const char* VariableDef, void* vPtr, DWORD ExprPtr, long vProperties);
   long  ucError(void);
   char* ucErrorData(long errIndex, long uThread);
   char* ucErrorMessage(long ErrorNumber, long uThread);
   long double ucEval(const char* Expression);
   char* ucEvalStr(const char* Expression);
   long double ucEvaluate(DWORD ExpressionPtr);
   long double ucEvaluate(DWORD ExpressionPtr, ucComplexNumberType& ucValue);
   long double ucEvaluateCplx(DWORD ExpressionPtr, ucComplexNumberType& ucValue);
   char* ucEvaluateStr(DWORD ExpressionPtr);
   char* ucGetErrorData(long errIndex, long uThread);
   long  ucGetThreadIndex(void);
   long double ucGetVariableValue(long VarHandle);
   char* ucGetVariableValueStr(long VarHandle);
   char* ucVariableValueStr(long VarHandle);
   long double ucImagPart(void);
   DWORD ucParse(const char* Expression, const char* LocalVars, long ExprType);
   long  ucPrecedence(const char* Op, long OpCategory);
   long  ucSymbolHandle(const char* ucSymbol, long SymbolProperty);

   long ucAlias(const char* AssignFrom, const char* AssignTo);
   void ucDetachSymbol(const char* Symbol);
   void ucErrorClear(void);
   char* ucErrorData(long errIndex);
   long ucErrorHandler(void *FunctionAddress);
   long ucPreParse(void *FunctionAddress);
   long ucTreeParseDownRaw(void *FunctionAddress);
   long ucTreeParseDown(void *FunctionAddress);
   long ucTreeParseUp(void *FunctionAddress);
   void ucReleaseItem(long ItemHandle);
   void ucReleaseExprPtr(DWORD ExpressionPtr);
   void ucSetThreadIndex(long uThread);
   void ucSetTrigMode(long TrigMode);
   void ucSetVariableValue(long VarHandle, long double vValue);
   void ucSetVariableValueStr(long VarHandle, const char* vValue);

private:
   long t;
   long ucNewThread(void);
   void ucReleaseThread(long t);
};

FastMath::FastMath()
{

//   ucThread = ucNewThread();
}

FastMath::~FastMath()
{
   ucReleaseThread(t);
}

long FastMath::ucDefineConstant(const char* ConstantDef, long vProperties = 0)
{
   return ucDefineVariableE(ConstantDef, 0, 0, vProperties, ucType_Constant, ucThreadAddr(t));
}

long FastMath::ucDefineFunction(const char* FunctionDef, void *FunctionAddress = 0, long fProperties = 0, long vType = ucType)
{
   return ucDefineFunctionE(FunctionDef, reinterpret_cast<DWORD>(FunctionAddress), 0, fProperties, vType, ucThreadAddr(t));
}

long FastMath::ucDefineOperator(const char* OperatorDef, long Precedence = 0, void *OperatorAddress = 0, long fProperties = 0, long vType = ucType)
{
   return ucDefineFunctionE(OperatorDef, reinterpret_cast<DWORD>(OperatorAddress), Precedence, fProperties, vType, ucThreadAddr(t));
}

long FastMath::ucDefineVariable(const char* VariableDef, long vProperties = 0)
{
   return ucDefineVariableE(VariableDef, 0, 0, vProperties, ucType_Internal, ucThreadAddr(t));
}

long FastMath::ucAttachVariable(const char* VariableDef, void* vPtr, DWORD ExprPtr = 0, long vProperties = 0)
{
   return ucDefineVariableE(VariableDef, reinterpret_cast<DWORD>(vPtr), ExprPtr, vProperties, ucType, ucThreadAddr(t));
}

long FastMath::ucError(void)
{
   ucThreadType tData;
   ucGetThreadInfo(t, tData);
   return tData.ErrorNumber;
}

char* FastMath::ucErrorData(long errIndex)
{
   return ucGetErrorDataE(errIndex, ucThreadAddr(t));
}

long double FastMath::ucEval(const char* Expression)
{
   return ucEvalE(Expression, ucThreadAddr(t));
}

char* FastMath::ucEvalStr(const char* Expression)
{
   return ucEvalStrE(Expression, ucThreadAddr(t));
}

long double FastMath::ucEvaluate(DWORD ExpressionPtr)
{
   return ucEvaluateE(ExpressionPtr);
}

long double FastMath::ucEvaluate(DWORD ExpressionPtr, ucComplexNumberType& ucValue)
{
   long double ucResult;

   ucResult = ucEvaluateE(ExpressionPtr);
   ucValue.Real = ucResult;
   ucValue.Imag = ucImagPartE(ucThreadAddr(t));

   return ucResult;
}

long double FastMath::ucEvaluateCplx(DWORD ExpressionPtr, ucComplexNumberType& ucValue)
{
   long double ucResult;

   ucResult = ucEvaluateE(ExpressionPtr);
   ucValue.Real = ucResult;
   ucValue.Imag = ucImagPartE(ucThreadAddr(t));

   return ucResult;
}

char* FastMath::ucEvaluateStr(DWORD ExpressionPtr)
{
   return ucEvaluateStrE(ExpressionPtr);
}

long FastMath::ucGetThreadIndex(void)
{
   return t;
}

long double FastMath::ucGetVariableValue(long VarHandle)
{
   return ucGetVariableValueE(VarHandle);
}

char* FastMath::ucGetVariableValueStr(long VarHandle)
{
   return ucGetVariableValueStrE(VarHandle);
}

char* FastMath::ucVariableValueStr(long VarHandle)
{
   return ucGetVariableValueStrE(VarHandle);
}

long double FastMath::ucImagPart(void)
{
   return ucImagPartE(ucThreadAddr(t));
}

DWORD FastMath::ucParse(const char* Expression, const char* LocalVars = "", long ExprType = 0)
{
   return ucParseE(Expression, LocalVars, ExprType, ucThreadAddr(t));
}

long FastMath::ucPrecedence(const char* Op, long OpCategory = 0)
{
   return ucPrecedenceE(Op, OpCategory, ucThreadAddr(t));
}

long FastMath::ucSymbolHandle(const char* ucSymbol, long SymbolProperty = 0)
{
   return ucSymbolHandleE(ucSymbol, SymbolProperty, ucThreadAddr(t));
}

long FastMath::ucAlias(const char* AssignFrom, const char* AssignTo)
{
   return ucAliasE(AssignFrom, AssignTo, ucThreadAddr(t));
}

void FastMath::ucDetachSymbol(const char* Symbol)
{
   ucDetachSymbolE(Symbol, ucThreadAddr(t));
}

void FastMath::ucErrorClear(void)
{
   ucErrorClearE(ucThreadAddr(t));
}

long FastMath::ucErrorHandler(void *FunctionAddress)
{
   return ucSetCallbackE(reinterpret_cast<DWORD>(FunctionAddress), ucErrorCallback, ucThreadAddr(t));
}

long FastMath::ucNewThread(void)
{
   return ucNewThreadE();
}

long FastMath::ucPreParse(void *FunctionAddress)
{
   return ucSetCallbackE(reinterpret_cast<DWORD>(FunctionAddress), ucTreePre, ucThreadAddr(t));
}

long FastMath::ucTreeParseDownRaw(void *FunctionAddress)
{
   return ucSetCallbackE(reinterpret_cast<DWORD>(FunctionAddress), ucTreeDownRaw, ucThreadAddr(t));
}

long FastMath::ucTreeParseDown(void *FunctionAddress)
{
   return ucSetCallbackE(reinterpret_cast<DWORD>(FunctionAddress), ucTreeDown, ucThreadAddr(t));
}

long FastMath::ucTreeParseUp(void *FunctionAddress)
{
   return ucSetCallbackE(reinterpret_cast<DWORD>(FunctionAddress), ucTreeUp, ucThreadAddr(t));
}

void FastMath::ucReleaseItem(long ItemHandle)
{
   ucReleaseItemE(ItemHandle);
}

void FastMath::ucReleaseExprPtr(DWORD ExpressionPtr)
{
   ucReleaseExprPtrE(ExpressionPtr);
}

void FastMath::ucReleaseThread(long uThread)
{
   if (uThread > 0) ucReleaseThreadE(uThread);
}

void FastMath::ucSetThreadIndex(long uThread)
{
   if (uThread == 0) ucReleaseThread(t);
   t = uThread;
}

void FastMath::ucSetTrigMode(long TrigMode)
{
   ucSetTrigModeE(TrigMode, ucThreadAddr(t));
}

void FastMath::ucSetVariableValue(long VarHandle, long double vValue)
{
   ucSetVariableValueE(VarHandle, vValue);
}

void FastMath::ucSetVariableValueStr(long VarHandle, const char* vValue)
{
   ucSetVariableValueStrE(VarHandle, vValue);
}


extern FastMath ucalc;

#endif /* __UCALC__ */

