' -------------------------------------------------------------------------
' ucalcpb.bas : PowerBASIC module file
' Designed to work with PB/DLL 6.1 or above, and PB/CC 2.1 or above
'
' UCalc Fast Math Parser (Beta J)
' Copyright (c) 1998-2004 by Daniel Corbier.  All rights reserved.
' http://www.ucalc.com/mathparser
' -------------------------------------------------------------------------

'Option Explicit '+++

Type ucDefinitionSpaceType
   BaseDefinitionSpace  As Long
   DefinitionSpace      As Long
   InlineBuffer         As Long
   MaxErrorData         As Long
   MaxThreadCount       As Long
End Type

Type ucForInternalUseType
   LanguageID           As Long
   ucOperator           As Dword
   ucEvalAddr           As Dword
   ucEvaluateAddr       As Dword
   ucThreadAddr         As Dword
   ParamListAddr        As Dword
   HandleAddr           As Dword
   ReleaseExprPtr       As Dword
   TreeIndex            As Long
   TreeParamInfoAddress As Dword
   HaltEvaluation       As Long
   ThreadStatus         As Long
   PrevSCount           As Long
   PrevLCount           As Long
   CompareOld           As Long
   IsStringValue        As Long
   ucStack              As Long
   Infinity             As Long
   TrigFactor           As Ext Ptr
   TrigFactorInv        As Ext Ptr
   im                   As String Ptr
End Type

Type ucPropertyType
   DefaultNumericType   As Long
   DefaultStringType    As Long
   FPU_ControlWord_UCalc       As Long
   FPU_ControlWord_HostProgram As Long
   CaseSensitive       As Long
   ComplexNumbers      As Long
   ImagSymbol          As Long
   RelationalTrue      As Long
   ImplicitMultiply    As Long
   MaxParamPerFunction As Long
   ArrayProperties     As Long
   FunctionProperties  As Long
   StackSize           As Long
   DefaultPrecedence   As Long
   RandomSeed          As Long
   OutputDigits        As Long
   QuoteChar           As Long
   QuoteCharSynonym    As Long
   ArgumentSeparator   As Long
   DecimalSeparator    As Long
   StatementSeparator  As Long
   DefSpace            As ucDefinitionSpaceType
End Type

Type ucHandleType
   ucThread       As Long
   Info           As Dword
   DataType       As Long
   DataPtr        As Dword ' +++
   DataCplxPtr    As Dword
   SymbolName     As String Ptr
   SortedHandle   As Long
   OverloadHandle As Long
   OpPrecedence   As Long
   ReqParamCount  As Integer  ' +++ Update other include files
   ParamCount     As Integer
   CallbackPtr    As Dword
   FuncArgs       As String Ptr
   FuncArgHandleData As String Ptr
   VarValueStr    As String Ptr ' +++ Consolidate with VarValue

   UserData1      As Dword
   UserData2      As Dword
   UserData3      As Dword
   UserData4      As Dword
End Type

Type ucThreadType
   ucThread             As Long
   Context              As Long
   ParamCount           As Long
   ErrorNumber          As Long
   TreeDirection        As Long
   DefSpace             As Long
   scount               As Long
   lcount               As Long
   Sequence             As Long
   SequenceLast         As Long
   ExprHasChanged       As Long
   IsFunctionParse      As Long
   ComplexStatus        As Long
   ucParamAddr          As Dword
   ucParamHandleAddr    As Dword
   SymbolStart          As Long
   SymbolEnd            As Long
   PrecedenceMax        As Long
   PrecedenceMin        As Long
   VarHandle            As Long
   OpHandle             As Long
   ReturnString         As String Ptr
   ReturnValue          As Ext Ptr
   ReturnComplex        As Ext Ptr
   ucProperty           As ucPropertyType
   Handshake            As ucForInternalUseType
   User1                As Long
   User2                As Long
   User3                As Long
   User4                As Long
End Type

Type ucComplexNumberType
   Real As Ext
   Imag As Ext
End Type

%ucTrue  = 1
%ucFalse = 0

' LanguageID
%uc_Lang_Unsupported    = 0
%uc_Lang_Debug          = 1
%uc_Lang_PowerBasic     = 2
%uc_Lang_Delphi         = 3
%uc_Lang_VisualBasic    = 4
%uc_Lang_VisualBasicNET = 5
%uc_Lang_CPlusPlus      = 6

' Type constants for vType arg of ucDefineFunction and ucDefineVariable
'[[+++ Revise this list later]]
%ucType_None          = -2
%ucType_Constant      =-1
%ucType_Internal      = 0
%ucType_Integer8      = 1
%ucType_Integer8u     = 2
%ucType_Integer16     = 3
%ucType_Integer16u    = 4
%ucType_Integer32     = 5
%ucType_Integer32u    = 6
%ucType_Integer64     = 7
%ucType_Integer64u    = 8
%ucType_Currency      = 9
%ucType_CurrencyExt   = 10
%ucType_FloatSingle   = 11
%ucType_FloatDouble   = 12
%ucType_FloatExtended = 13
%ucType_StringBSTR    = 14  ' For VB
%ucType_StringLPSTR   = 15  ' (ASCIIZ) C++ char default
%ucType_StringAnsi    = 16  ' PowerBASIC and Delphi default

%ucType_Long          = %ucType_Integer32
%ucType_Double        = %ucType_FloatDouble
%ucType_Extended      = %ucType_FloatExtended
%ucType_AnsiString    = %ucType_StringAnsi

' Context options (t.Context)
%ucContext_ucEval            = 1
%ucContext_ucEvalStr         = 2
%ucContext_ucEvaluateStr     = 4
%ucContext_ucParse           = 8
%ucContext_ucParseOrucEval   = 16
%ucContext_ucDefineFunction  = 32
%ucContext_ucDefineVariable  = 64
%ucContext_ucReset           = 128
%ucContext_MultiLineDef      = 256
%ucContext_MultiLineDefFirst = 512
%ucContext_MultiLineDefLast  = 1024

' Trig mode options for ucSetTrigMode
%ucRadians = 1
%ucDegrees = 2
%ucGrads   = 3
%ucCycles  = 4

' Options for ucArrayInfo,
' and the first two for ucProperties.ArrayProperties
%ucArray_BoundsCheck    = 1
%ucArray_ColumnMajor    = 2
%ucArray_DataAddress    = 3
%ucArray_DataType       = 4
%ucArray_DimensionCount = 5
%ucArray_ElementCount   = 6
%ucArray_ElementSize    = 7

' Options for fProperties parameter of ucDefineFunction
' and for and for ucProperties.FunctionProperties
%ucInline       = 1
%ucDynamic      = 2
%ucFastCallback = 4
%ucLocal        = 8
%ucComplex      = 16
%ucLocked       = 32
%ucConstant     = %ucLocked
%ucOverride     = 64
%ucOverwrite    = 128
%ucOverload     = 256
%ucRecursion    = 512
%ucBootstrap    = 1024
%ucMultiLine    = 2048
%ucEndMultiLine = %ucMultiLine

' FPU control word values for ucProperties.FPU_ControlWord,
' and SetFPU(), SetOrFPU, SetXorFPU, GetFPU(), RestoreFPU
%ucFPU_ErrMask_InvalidOp     = 1
%ucFPU_ErrMask_DenormalOp    = 2
%ucFPU_ErrMask_ZeroDivide    = 4
%ucFPU_ErrMask_Overflow      = 8
%ucFPU_ErrMask_Underflow     = 16
%ucFPU_ErrMask_PrecisionLoss = 32

%ucFPU_Precision_Single      = 0
%ucFPU_Precision_Double      = 256
%ucFPU_Precision_Extended    = 768

%ucFPU_Round_Even            = 0
%ucFPU_Round_Down            = 1024
%ucFPU_Round_Up              = 2048
%ucFPU_Round_Toward          = 3072

' For ucTreeProperty, ucTreeNodeProperty,
' ucTreeParamProperty, and ucTreeParentProperty
%ucIsInline          = 0
%ucIsDynamic         = 1
%ucIsFastCallback    = 2
%ucIsLocal           = 3
%ucIsComplex         = 4
%ucIsLocked          = 5
%ucIsConstant        = %ucIsLocked
%ucIsOverride        = 6
%ucIsOverwrite       = 7
%ucIsOverload        = 8
%ucIsRecursion       = 9
%ucIsBootstrap       = 10
%ucIsMultiLine       = 11
%ucIsEndMultiLine    = %ucIsMultiLine

%ucIsArray           = 12

%ucIsFunction        = 13
%ucIsOperator        = 14
%ucIsBinaryOperator    = 15
%ucIsUnaryPostOperator = 16
%ucIsUnaryPreOperator  = 17
%ucIsVariable        = 18
%ucIsJITVariable     = 19
%ucIsJITExpression   = 20
%ucIsString          = 21
%ucIsInternal        = 22
%ucIsExternal        = 23
%ucIsActive          = 24
%ucIsLCount          = 25
%ucIsSorted          = 26
%ucIsMaybeComplex    = 27
%ucHasComplexAltDef  = 28
%PassAnArray         = 29
%Reserved1           = 30
%Reserved2           = 31

%ucBinaryOp  = %ucIsBinaryOperator
%ucUnaryPost = %ucIsUnaryPostOperator
%ucUnaryPre  = %ucIsUnaryPreOperator

' For ucElementCategory
%ucIsOperand           = 32
'%ucIsFunction          = (already defined)
%ucIsFunctionArg       = 33
'%ucIsBinaryOperator    = (already defined)
'%ucIsUnaryPreOperator  = (already defined)
'%ucIsUnaryPostOperator = (already defined)

%ucDataType    = 40
%ucResultPtr   = 41
%ucSignPos     = 42
%ucElementCat  = 43
%ucTreeIndex   = 44
%ucErrorNumber = 45
%ucOpHandle    = 46
%ucFuncHandle  = %ucOpHandle
%ucExprPtr     = 47
%ucCodeAddress = 48
%ucCurrentPtr  = 49
%ucParentPtr   = 50
%ucDataHandle  = 51
%ucMiscData    = 52
%ucUserData1   = 53
%ucUserData2   = 54
%ucUserData3   = 55
%ucUserData4   = 56

%ucTreePre  = 5
%ucTreeDownRaw = 6
%ucTreeDown = 7
%ucTreeUp   = 8

%ucErrorCallback = 9

%ucNoChange    = 0
%ucMinorChange = 1
%ucMajorChange = 2

' For ucSetErrorData / ucGetErrorData
%ucNewIndex = -2
%ucNewUniqueIndex  = -1
%ucAllErrorData    = -1

' Error values
%ucErr_None                = 0
%ucErr_ucErrorData         = 1
%ucErr_SyntaxError         = 2
%ucErr_MismatchedParenth   = 3
%ucErr_MismatchedQuotes    = 4
%ucErr_UndefinedVariable   = 5
%ucErr_UndefinedFunction   = 6
%ucErr_IncorrectArgType    = 7
%ucErr_InvalidNumberOfArgs = 8
%ucErr_InvalidOperandType  = 9
%ucErr_InvalidName         = 10
%ucErr_ImplicitMult        = 11

%ucErr_InvalidOperation    = 20
%ucErr_DenormalOperand     = 21
%ucErr_DivisionByZero      = 22
%ucErr_Overflow            = 23
%ucErr_Underflow           = 24
%ucErr_InexactResult       = 25
%ucErr_SubscriptOutOfRange = 35
%ucErr_StackOverflow       = 36

%ucErr_OperationDenied     = 40
%ucErr_PropertyConflict    = 41
%ucErr_ConstantWriteDenied = 42
%ucErr_FunctionDefError    = 43
%ucErr_FunctionLocked      = 44

%ucErr_DefinitionSpaceFull = 60
%ucErr_ThreadInitFailed    = 61
%ucErr_UnsupportedFeature  = 62

Global t0 As ucThreadType
Global ucInitialized As Long
Global ucItem() As ucHandleType

Declare Sub ucDetachSymbolE Lib "ucalcfmp.dll" Alias "ucDetachSymbolE" (Symbol As Asciiz, t As ucThreadType)
Declare Sub ucErrorClearE Lib "ucalcfmp.dll" Alias "ucErrorClearE" (ByVal t As Dword)
Declare Sub ucLicenseE Lib "ucalcfmp.dll" Alias "ucLicenseE" (SerialNumber As Asciiz)
Declare Sub ucReleaseItemE Lib "ucalcfmp.dll" Alias "ucReleaseItemE" (ByVal ItemHandle As Long)
Declare Sub ucReleaseThreadE Lib "ucalcfmp.dll" Alias "ucReleaseThreadE" (ByVal ucThread As Long)
Declare Sub ucResetE Lib "ucalcfmp.dll" Alias "ucResetE" (t As ucThreadType)
Declare Sub ucReturnE Lib "ucalcfmp.dll" Alias "ucReturnE" (ByVal vValue As Ext, t As ucThreadType)
Declare Sub ucReturnCplxE Lib "ucalcfmp.dll" Alias "ucReturnCplxE" (ByVal vValue As Ext, t As ucThreadType)
Declare Sub ucReturnStrE Lib "ucalcfmp.dll" Alias "ucReturnStrE" (Expression As Asciiz, t As ucThreadType)
Declare Sub ucSetErrorDataE Lib "ucalcfmp.dll" Alias "ucSetErrorDataE" (ByVal errIndex As Long, ByVal vValue As String, t As ucThreadType)
Declare Sub ucSetParamStrE Lib "ucalcfmp.dll" Alias "ucSetParamStrE" (ByVal ParamIndex As Long, Argument As Asciiz, t As ucThreadType)
Declare Sub ucSetThreadAddrE Lib "ucalcfmp.dll" Alias "ucSetThreadAddrE" (ByVal ucThread As Long, t As ucThreadType)
Declare Sub ucSetTrigModeE Alias "ucSetTrigModeE" (ByVal TrigMode As Long, t As ucThreadType)
Declare Sub ucSetVariableValueE Lib "ucalcfmp.dll" Alias "ucSetVariableValueE" (ByVal VarHandle As Long, ByVal vValue As Ext)
Declare Sub ucSetVariableValueStrE Lib "ucalcfmp.dll" Alias "ucSetVariableValueStrE" (ByVal VarHandle As Long, ByVal vValue As String)
Declare Sub ucTreeSetParamE Lib "ucalcfmp.dll" Alias "ucTreeSetParamE" (ByVal pIndex As Long, pData As Asciiz, t As ucThreadType)
Declare Function ucAliasE Lib "ucalcfmp.dll" Alias "ucAliasE" (AssignFrom As Asciiz, AssignTo As Asciiz, ByVal t As Dword) As Long
Declare Function ucArrayInfoE Lib "ucalcfmp.dll" Alias "ucArrayInfoE" (ByVal VarHandle As Long, ByVal InfoOption As Long) As Dword
Declare Function ucDefineFunctionE Lib "ucalcfmp.dll" Alias "ucDefineFunctionE" (FunctionDef As Asciiz, ByVal FunctionAddress As Dword, ByVal Precedence As Long, ByVal fProperties As Long, ByVal vType As Long, ByVal t As Dword) As Long
Declare Function ucDefineVariableE Lib "ucalcfmp.dll" Alias "ucDefineVariableE" (VariableDef As Asciiz, ByVal vPtr As Dword, ByVal Expr As Dword, ByVal vProperties As Long, ByVal vType As Long, ByVal t As Dword) As Long
Declare Function ucEvalE Lib "ucalcfmp.dll" Alias "ucEvalE" (Expression As Asciiz, ByVal t As Dword) As Ext
Declare Function ucEvalStrE Lib "ucalcfmp.dll" Alias "ucEvalStrE" (Expression As Asciiz, ByVal t As Dword) As String
Declare Function ucEvaluateStrE Lib "ucalcfmp.dll" Alias "ucEvaluateStrE" (ByVal ExpressionPtr As Long) As String
Declare Function ucGetErrorDataE Lib "ucalcfmp.dll" Alias "ucGetErrorDataE" (ByVal errIndex As Long, ByVal t As Dword) As String
Declare Function ucGetParamE Lib "ucalcfmp.dll" Alias "ucGetParamE" (ByVal pIndex As Long, t As ucThreadType) As Ext
Declare Function ucGetParamCplxE Lib "ucalcfmp.dll" Alias "ucGetParamCplxE" (ByVal pIndex As Long, t As ucThreadType) As Ext
Declare Function ucGetParamStrE Lib "ucalcfmp.dll" Alias "ucGetParamStrE" (ByVal ParamIndex As Long, t As ucThreadType) As String
Declare Function ucGetThreadInfoE Lib "ucalcfmp.dll" Alias "ucGetThreadInfoE" (ByVal ucThread As Long, t As ucThreadType) As Dword
Declare Function ucGetVariableValueE Lib "ucalcfmp.dll" Alias "ucGetVariableValueE" (ByVal VarHandle As Long) As Ext
Declare Function ucGetVariableValueStrE Lib "ucalcfmp.dll" Alias "ucGetVariableValueStrE" (ByVal VarHandle As Long) As String
Declare Function ucImagPartE Lib "ucalcfmp.dll" Alias "ucImagPartE" (ByVal t As Dword) As Ext
Declare Function ucLowerBoundE Lib "ucalcfmp.dll" Alias "ucLowerBoundE" (ByVal VarHandle As Long, ByVal Dimension As Long) As Long
Declare Function ucNewThreadE Lib "ucalcfmp.dll" Alias "ucNewThreadE" () As Long
Declare Function ucNewSymbolHandleE Lib "ucalcfmp.dll" Alias "ucNewSymbolHandleE" (t As ucThreadType) As Long
Declare Function ucParamHandleE Lib "ucalcfmp.dll" Alias "ucParamHandleE" (ByVal pIndex As Long, t As ucThreadType) As Long
Declare Function ucParamTypeE Lib "ucalcfmp.dll" Alias "ucParamTypeE" (ByVal pIndex As Long, t As ucThreadType) As Long
Declare Function ucParseE Lib "ucalcfmp.dll" Alias "ucParseE" (Expression As Asciiz, LocalVars As Asciiz, ByVal ExprType As Long, ByVal t As Dword) As Dword
Declare Function ucPrecedenceE Lib "ucalcfmp.dll" Alias "ucPrecedenceE" (Op As Asciiz, ByVal OpCategory As Long, ByVal t As Dword) As Long
Declare Function ucSymbolHandleE Lib "ucalcfmp.dll" Alias "ucSymbolHandleE" (ucSymbol As Asciiz, ByVal SymbolProperty As Long, ByVal t As Dword) As Long
Declare Function ucThreadAddrE Lib "ucalcfmp.dll" Alias "ucThreadAddrE" (ByVal t As Long) As Dword
Declare Function ucTreeGetParamE Lib "ucalcfmp.dll" Alias "ucTreeGetParamE" (ByVal pIndex As Long, t As ucThreadType) As String
Declare Function ucTreePropertyE Lib "ucalcfmp.dll" Alias "ucTreePropertyE" (ByVal NodePtr As Dword, ByVal pIndex As Long, ByVal pOption As Long, t As ucThreadType) As Long
Declare Function ucSetCallbackE Lib "ucalcfmp.dll" Alias "ucSetCallbackE" (ByVal FunctionAddress As Dword, ByVal TreeOption As Long, ByVal t As Dword) As Long
Declare Function ucUpperBoundE Lib "ucalcfmp.dll" Alias "ucUpperBoundE" (ByVal VarHandle As Long, ByVal Dimension As Long) As Long

' These 3 functions come from ucalcfmE.dll instead of ucalcfmP.dll
Declare Function ucEvaluate Lib "ucalcfme.dll" Alias "ucEvaluateE" (ByVal ExpressionPtr As Dword) As Ext
Declare Sub ucEvaluateInitE Lib "ucalcfme.dll" Alias "ucEvaluateInitE" ()
Declare Sub ucReleaseExprPtrE Lib "ucalcfme.dll" Alias "ucReleaseExprPtrE" (ByVal ExpressionPtr As Dword)

' +++ Have alternate #If for multithread and one for single.

Function ucThreadAddr(ByVal t As Long) As Dword
   If ucInitialized = %ucFalse Then Call ucEvaluateInit
   ucThreadAddr = ucThreadAddrE(t)
End Function

Function ucAlias(AssignFrom$, AssignTo$, Optional ByVal t As Long) As Long
   ucAlias = ucAliasE(AssignFrom$+Chr$(0), AssignTo$+Chr$(0), ucThreadAddr(t))
End Function

Sub ucDetachSymbol(Symbol$, t As ucThreadType)
   ucDetachSymbolE Symbol$+Chr$(0), t
End Sub

Sub ucErrorClear(Optional ByVal t As Long)
   ucErrorClearE ucThreadAddr(t)
End Sub

Sub ucReleaseExprPtr(ByVal ExpressionPtr As Dword)
   ucReleaseExprPtrE ExpressionPtr
End Sub

Sub ucReleaseItem(ByVal ItemHandle As Long)
   ucReleaseItemE ItemHandle
End Sub

Sub ucReleaseThread(ByVal ucThread As Long)
   If ucThread > 0 Then ucReleaseThreadE ucThread
End Sub

Sub ucSetVariableValue(ByVal VarHandle As Long, ByVal vValue As Ext)
   ucSetVariableValueE VarHandle, vValue
End Sub

Sub ucSetVariableValueStr(ByVal VarHandle As Long, vValue$)
   ucSetVariableValueStrE VarHandle, vValue$+Chr$(0)
End Sub

Function ucArrayInfo(ByVal VarHandle As Long, ByVal InfoOption As Long) As Dword
   ucArrayInfo = ucArrayInfoE(VarHandle, InfoOption)
End Function

Function ucDefineConstant(ConstantDef$, Optional ByVal vProperties As Long, Optional ByVal t As Long) As Long
   ucDefineConstant = ucDefineVariableE(ConstantDef$+Chr$(0), 0, 0, vProperties, %ucType_Constant, ucThreadAddr(t))
End Function

Function ucDefineFunction(FunctionDef$, Optional ByVal FunctionAddress As Dword, Optional ByVal fProperties As Long, Optional ByVal vType As Long, Optional ByVal t As Long) As Long
   ucDefineFunction = ucDefineFunctionE(FunctionDef$+Chr$(0), FunctionAddress, 0, fProperties, vType, ucThreadAddr(t))
End Function

Function ucDefineOperator(OperatorDef$, Optional ByVal Precedence As Long, Optional ByVal OperatorAddress As Dword, Optional ByVal fProperties As Long, Optional ByVal vType As Long, Optional ByVal t As Long) As Long
   ucDefineOperator = ucDefineFunctionE(OperatorDef$+Chr$(0), OperatorAddress, Precedence, fProperties, vType, ucThreadAddr(t))
End Function

Function ucDefineVariable(VariableDef$, Optional ByVal vProperties As Long, Optional ByVal t As Long) As Long
   ucDefineVariable = ucDefineVariableE(VariableDef$+Chr$(0), 0, 0, vProperties, %ucType_Internal, ucThreadAddr(t))
End Function

Function ucAttachVariable(VariableDef$, ByRef vPtr As Dword, Optional ByVal ExprPtr As Dword, Optional ByVal vProperties As Long, Optional ByVal t As Long) As Long
   ucAttachVariable = ucDefineVariableE(VariableDef$+Chr$(0), vPtr, ExprPtr, vProperties, %ucType_Internal, ucThreadAddr(t))
End Function

Function ucErrorHandler(ByVal FunctionAddress As Dword, Optional ByVal t As Long) As Long
   ucErrorHandler = ucSetCallbackE(FunctionAddress, %ucErrorCallback, ucThreadAddr(t))
End Function

Function ucEval(Expression$, Optional ByVal t As Long) As Ext
   ucEval = ucEvalE(Expression$+Chr$(0), ucThreadAddr(t))
End Function

Function ucEvalStr(Expression$, Optional ByVal t As Long) As String
   ucEvalStr = ucEvalStrE(Expression$+Chr$(0), ucThreadAddr(t))
End Function

'Macro ucEvaluate(ExpressionPtr) = ucEvaluateE(ExpressionPtr)

'Function ucEvaluate(ByVal ExpressionPtr As Dword) As Ext
'   ucEvaluate = ucEvaluateE(ExpressionPtr)
'End Function

Function ucEvaluateStr(ByVal ExpressionPtr As Dword) As String
   ucEvaluateStr = ucEvaluateStrE(ExpressionPtr)
End Function

Function ucError() As Long
   ucError = t0.ErrorNumber
End Function

Function ucGetErrorData(ByVal errIndex As Long, Optional ByVal t As Long) As String
   ucGetErrorData = ucGetErrorDataE(errIndex, ucThreadAddr(t))
End Function

Function ucGetVariableValue(ByVal VarHandle As Long) As Ext
   ucGetVariableValue = ucGetVariableValueE(VarHandle)
End Function

Function ucGetVariableValueStr(ByVal VarHandle As Long) As String
   ucGetVariableValueStr = ucGetVariableValueStrE(VarHandle)
End Function

Function ucImagPart(Optional ByVal t As Long) As Ext
   ucImagPart = ucImagPartE(ucThreadAddr(t))
End Function

Function ucLowerBound(ByVal VarHandle As Long, Optional ByVal Dimension As Long) As Long
   If Dimension = 0 Then Dimension = 1
   ucLowerBound = ucLowerBoundE(VarHandle, Dimension)
End Function

Function ucNewThread() As Long
   ucNewThread = ucNewThreadE
End Function

Function ucNewSymbolHandle(t As ucThreadType) As Long
   ucNewSymbolHandle = ucNewSymbolHandleE(t)
End Function

Function ucParse(Expression$, Optional ByVal LocalVars$, Optional ByVal ExprType As Long, Optional ByVal t As Long) As Dword
   ucParse = ucParseE(Expression$+Chr$(0), LocalVars$+Chr$(0), ExprType, ucThreadAddr(t))
End Function

Function ucPrecedence(Op$, Optional ByVal OpCategory As Long, Optional ByVal t As Long) As Long
   ucPrecedence = ucPrecedenceE(Op$+Chr$(0), OpCategory, ucThreadAddr(t))
End Function

Function ucSymbolHandle(ByVal ucSymbol$, Optional ByVal SymbolProperty As Long, Optional ByVal t As Long) As Long
   ucSymbolHandle = ucSymbolHandleE(ucSymbol$+Chr$(0), SymbolProperty, ucThreadAddr(t))
End Function

Function ucErrorData(ByVal errIndex As Long, Optional ByVal t As Long) As String
   ucErrorData = ucGetErrorDataE(errIndex, ucThreadAddr(t))
End Function

Function ucUpperBound(ByVal VarHandle As Long, Optional ByVal Dimension As Long) As Long
   If Dimension = 0 Then Dimension = 1
   ucUpperBound = ucUpperBoundE(VarHandle, Dimension)
End Function

Function ucVariableValue(ByVal VarHandle As Long) As Ext
   ucVariableValue = ucGetVariableValueE(VarHandle)
End Function

Function ucVariableValueStr(ByVal VarHandle As Long) As String
   ucVariableValueStr = ucGetVariableValueStrE(VarHandle)
End Function

Function ucGetThreadInfo(ByVal ucThread As Long, t As ucThreadType) As Dword
   ucGetThreadInfo = ucGetThreadInfoE(ucThread, t)
End Function
Sub ucSetThreadAddr(ByVal ucThread As Long, t As ucThreadType)
   ucSetThreadAddrE ucThread, t
End Sub

Sub ucSetTrigMode(ByVal TrigMode As Long, t As ucThreadType)
   ucSetTrigMode TrigMode, t
End Sub

Sub ucReturn(ByVal vValue As Ext, t As ucThreadType)
   ucReturnE vValue, t
End Sub

' +++ Add all other new exported routines and for other include files
Sub ucReturnCplx(ByVal vValue As Ext, t As ucThreadType)
   ucReturnCplxE vValue, t
End Sub

Sub ucReturnStr(Expression$, t As ucThreadType)
   ucReturnStrE Expression$+Chr$(0), t
End Sub

Function ucGetParamStr(ByVal ParamIndex As Long, t As ucThreadType) As String
   ucGetParamStr = ucGetParamStrE(ParamIndex, t)
End Function
Sub ucSetParamStr(ByVal ParamIndex As Long, Argument$, t As ucThreadType)
   ucSetParamStrE ParamIndex, Argument$+Chr$(0), t
End Sub

Function ucParamStr(ByVal ParamIndex As Long, t As ucThreadType) As String
   ucParamStr = ucGetParamStrE(ParamIndex, t)
End Function

Sub ucLicense(SerialNumber$)
   ucLicenseE SerialNumber$+Chr$(0)
End Sub

Function ucElementCategory(t As ucThreadType) As Long
   ucElementCategory = ucTreePropertyE(0, 0, %ucElementCat, t)
End Function

Function ucTreeProperty(ByVal NodePtr As Dword, ByVal pIndex As Long, ByVal pOption As Long, t As ucThreadType) As Long
   ucTreeProperty = ucTreePropertyE(NodePtr, pIndex, pOption, t)
End Function

Function ucTreeNodeProperty(ByVal pOption As Long, t As ucThreadType) As Long
   ucTreeNodeProperty = ucTreePropertyE(0, 0, pOption, t)
End Function

Function ucTreeParamProperty(ByVal pIndex As Long, ByVal pOption As Long, t As ucThreadType) As Long
   ucTreeParamProperty = ucTreePropertyE(0, pIndex, pOption, t)
End Function

Function ucTreeParentProperty(ByVal pOption As Long, t As ucThreadType) As Long
   Dim ParentPtr As Dword

   ParentPtr = ucTreePropertyE(0, 0, %ucParentPtr, t)
   ucTreeParentProperty = ucTreePropertyE(ParentPtr, 0, pOption, t)
End Function

Function ucPreParse(ByVal FunctionAddress As Dword, Optional ByVal t As Long) As Long
   ucPreParse = ucSetCallbackE(FunctionAddress, %ucTreePre, ucThreadAddr(t))
End Function

Function ucTreeParseDownRaw(ByVal FunctionAddress As Dword, Optional ByVal t As Long) As Long
   ucTreeParseDownRaw = ucSetCallbackE(FunctionAddress, %ucTreeDownRaw, ucThreadAddr(t))
End Function

Function ucTreeParseDown(ByVal FunctionAddress As Dword, Optional ByVal t As Long) As Long
   ucTreeParseDown = ucSetCallbackE(FunctionAddress, %ucTreeDown, ucThreadAddr(t))
End Function

Function ucTreeParseUp(ByVal FunctionAddress As Dword, Optional ByVal t As Long) As Long
   ucTreeParseUp = ucSetCallbackE(FunctionAddress, %ucTreeUp, ucThreadAddr(t))
End Function

Function ucGetExpr(t As ucThreadType) As String
   ucGetExpr = ucTreeGetParamE(-1, t)
End Function

Function ucTreeGetExpr(t As ucThreadType) As String
   ucTreeGetExpr = ucTreeGetParamE(-1, t)
End Function

Function ucTreeGetFunc(t As ucThreadType) As String
   ucTreeGetFunc = ucTreeGetParamE(0, t)
End Function

Function ucTreeGetOp(t As ucThreadType) As String
   ucTreeGetOp = ucTreeGetParamE(0, t)
End Function

Function ucTreeGetParam(ByVal pIndex As Long, t As ucThreadType) As String
   ucTreeGetParam = ucTreeGetParamE(pIndex, t)
End Function

Function ucTreeGetParamList(t As ucThreadType) As String
   ucTreeGetParamList = ucTreeGetParamE(-2, t)
End Function
' +++ All ucTreeGet should have synonym w/o Get

Sub ucSetExpr(Expression$, t As ucThreadType)
   ucTreeSetParamE -1, Expression$+Chr$(0), t
End Sub

Sub ucTreeSetExpr(Expression$, t As ucThreadType)
   ucTreeSetParamE -1, Expression$+Chr$(0), t
End Sub

Sub ucTreeSetFunc(TreeFunc$, t As ucThreadType)
   ucTreeSetParamE 0, TreeFunc$+Chr$(0), t
End Sub

Sub ucTreeSetOp(TreeOp$, t As ucThreadType)
   ucTreeSetParamE 0, TreeOp$+Chr$(0), t
End Sub

Sub ucTreeSetParam(ByVal pIndex As Long, pData$, t As ucThreadType)
   ucTreeSetParamE pIndex, pData$+Chr$(0), t
End Sub

Sub ucTreeSetParamList(pData$, t As ucThreadType)
   ucTreeSetParamE -2, pData$+Chr$(0), t
End Sub

Function ucParam(ByVal x As Long, t As ucThreadType) As Ext
   ucParam = ucGetParamE(x, t)
End Function

Function ucParamCplx(ByVal x As Long, t As ucThreadType) As Ext
   ucParamCplx = ucGetParamCplxE(x, t)
End Function

Function ucParamHandle(ByVal pIndex As Long, t As ucThreadType) As Long
   ucParamHandle = ucParamHandleE(pIndex, t)
End Function

Function ucParamType(ByVal pIndex As Long, t As ucThreadType) As Long
   ucParamType = ucParamTypeE(pIndex, t)
End Function

Sub ucSetErrorData(ByVal errIndex As Long, vValue$, t As ucThreadType)
   ucSetErrorDataE errIndex, vValue$+Chr$(0), t
End Sub


' *** String operators: =, <>, >, >=, <, <=, +
Function ucEqual(t As ucThreadType) As Ext
    ucEqual = t.ucProperty.RelationalTrue * Abs(ucParamStr(1, t) = ucParamStr(2, t))
End Function

Function ucNotEqual(t As ucThreadType) As Ext
    ucNotEqual = t.ucProperty.RelationalTrue * Abs(ucParamStr(1, t) <> ucParamStr(2, t))
End Function

Function ucGreater(t As ucThreadType) As Ext
    ucGreater = t.ucProperty.RelationalTrue * Abs(ucParamStr(1, t) > ucParamStr(2, t))
End Function

Function ucGreaterOrEqual(t As ucThreadType) As Ext
    ucGreaterOrEqual = t.ucProperty.RelationalTrue * Abs(ucParamStr(1, t) >= ucParamStr(2, t))
End Function

Function ucLess(t As ucThreadType) As Ext
    ucLess = t.ucProperty.RelationalTrue * Abs(ucParamStr(1, t) < ucParamStr(2, t))
End Function

Function ucLessOrEqual(t As ucThreadType) As Ext
    ucLessOrEqual = t.ucProperty.RelationalTrue * Abs(ucParamStr(1, t) <= ucParamStr(2, t))
End Function

Function ucEvaluateCplx(ByVal ExpressionPtr As Dword, ucValue As ucComplexNumberType, t As ucThreadType) As Ext
   ucValue.Real = ucEvaluate(ExpressionPtr)
   ucValue.Imag = ucImagPartE(ucThreadAddr(t.ucThread))
   ucEvaluateCplx = ucValue.Real
End Function

#If Not %Def(%ucAddOnLibrary)
#If Not %Def(%ucAltErrMsg)
Function ucErrorMessage(Optional ByVal ErrorNumber As Long, Optional ByVal ucThread As Long) As String
   Dim ErrMsg As String

   If ErrorNumber = 0 Then ErrorNumber = t0.ErrorNumber

   Select Case ErrorNumber
      Case 0: ' No Error
      Case 1:  ErrMsg = ucErrorData(0, ucThread)
      Case 2:  ErrMsg = "Error #2: Syntax error"
      Case 3:  ErrMsg = "Error #3: Mismatched parenthesis"
      Case 4:  ErrMsg = "Error #4: Mismatched quotes"
      Case 5:  ErrMsg = "Error #5: Undefined variable"
      Case 6:  ErrMsg = "Error #6: Undefined function"
      Case 7:  ErrMsg = "Error #7: Incorrect argument type"
      Case 8:  ErrMsg = "Error #8: Invalid number of function arguments"
      Case 9:  ErrMsg = "Error #9: Invalid operand type"
      Case 10: ErrMsg = "Error #10: Invalid name"
      Case 11: ErrMsg = "Error #11: Implicit multiplication error"

      Case 20: ErrMsg = "Error #20: Invalid floating point operation"
      Case 21: ErrMsg = "Error #21: Denormal floating point operand"
      Case 22: ErrMsg = "Error #22: Division by 0"
      Case 23: ErrMsg = "Error #23: Floating point overflow"
      Case 24: ErrMsg = "Error #24: Floating point underflow"
      Case 25: ErrMsg = "Error #25: Inexact floating point result"
      Case 35: ErrMsg = "Error #35: Array subscript out of range"
      Case 36: ErrMsg = "Error #36: Stack overflow"

      Case 40: ErrMsg = "Error #40: Cannot perform requested operation"
      Case 41: ErrMsg = "Error #41: Definition property conflict"
      Case 42: ErrMsg = "Error #42: Assignment to a constant not allowed"
      Case 43: ErrMsg = "Error #43: Invalid function or operator definition"
      Case 44: ErrMsg = "Error #44: Function or operator is locked"

      Case 60: ErrMsg = "Error #60: Definition space is full"
      Case 61: ErrMsg = "Error #61: Thread initialization failed"
      Case 62: ErrMsg = "Error #62: Requires advanced edition of UCalc FMP"
      Case Else: ErrMsg = "Unknown error"
      ' // numbers up to 500 are reserved for future use by UCalc
      ' // Use Case 501 and beyond for user-defined errors
   End Select
   ucErrorMessage = ErrMsg
End Function
#EndIf

Sub ucReset(t As ucThreadType)
   Dim TempContext As Long
   Dim Between_Mult_Pow As Long
   Dim Between_And_Less As Long

   ReDim ucItem(t.ucProperty.DefSpace.BaseDefinitionSpace _
               +t.ucProperty.DefSpace.DefinitionSpace     _
               *t.ucProperty.DefSpace.MaxThreadCount) As ucHandleType

   TempContext = t.Context
   t.Context = t.Context Or %ucContext_ucReset
   t.Handshake.LanguageID = %uc_Lang_PowerBASIC
   t.Handshake.HandleAddr = VarPtr(ucItem(0))
   ucEvaluateInitE()
   ucResetE t
   t.ucProperty.ComplexNumbers = %ucTrue
   t.ucProperty.FunctionProperties = %ucInline + %ucLocked + %ucOverride + %ucOverload
   Between_Mult_Pow = (ucPrecedence("*") + ucPrecedence("^")) / 2
   Between_And_Less = (ucPrecedence("AND") + ucPrecedence("<")) / 2

   ' String relational (comparison) operators
   ucDefineOperator " =[a$,b$]", ucPrecedence("="),  CodePtr(ucEqual)
   ucDefineOperator "<>[a$,b$]", ucPrecedence("<>"), CodePtr(ucNotEqual)
   ucDefineOperator " >[a$,b$]", ucPrecedence(">"),  CodePtr(ucGreater)
   ucDefineOperator ">=[a$,b$]", ucPrecedence(">="), CodePtr(ucGreaterOrEqual)
   ucDefineOperator " <[a$,b$]", ucPrecedence("<"),  CodePtr(ucLess)
   ucDefineOperator "<=[a$,b$]", ucPrecedence("<="), CodePtr(ucLessOrEqual)

   ' Alternative notations
   ' +++ Need to document that & and | are no longer supported as built-in ops.
'   ucDefineOperator "  &[a,b] = a and b", ucPrecedence("and") ' +++ Document the removal of these 4 alternative ops
'   ucDefineOperator "  |[a,b] = a or  b", ucPrecedence("or ")
'   ucDefineOperator " =<[a,b] = a <=  b", ucPrecedence("<= ")
'   ucDefineOperator " =>[a,b] = a >=  b", ucPrecedence(">= ")
   ucDefineOperator " ==[a,b] = a  =  b", ucPrecedence("=")
   ucDefineOperator "DIV[a,b] = a  \  b", ucPrecedence("\")
   ucDefineOperator " **[a,b] = a  ^  b", ucPrecedence("^")
   ucDefineOperator " &$[TextA$,TextB$] = TextA + TextB", ucPrecedence("+")
   ucDefineOperator "==[TextA$,TextB$] = TextA = TextB", ucPrecedence("=")
   ucDefineFunction "Concatenate$(a$,b$) = a + b"
   ucDefineFunction "RND()   = RND(1)"
   ucDefineFunction "Sign(x) = Sgn(x)"
   ucDefineFunction "Sqrt(x) = Sqr(x)"
   ucDefineFunction "Ln(x)   = Log(x)"
   ucDefineFunction "Atan(x) = Atn(x)" ' +++ should do these after complex defs

   ucDefineOperator "[x]! = fact(x)", t.PrecedenceMax
   ucDefineOperator "[x]% = x/100", t.PrecedenceMax
   ucDefineOperator "%[x,y] = x mod y", ucPrecedence("MOD")
   ' +++ Document the subtle difference in the order of definition for things like %
   ucDefineOperator "-[x] = 0-x", Between_Mult_Pow
   ucDefineOperator "+[x] = 0+x", Between_Mult_Pow
   ucDefineOperator "Not[x] = Not(x)", Between_And_Less

   ' FPU routines
   ucDefineFunction "GetFPU() = GetFPU(0)"
   ucDefineFunction "RestoreFPU() = RestoreFPU(0)"
   ucAlias "GetFPU", "GetFPU(0)"
   ucAlias "RestoreFPU", "RestoreFPU(0)"
   ucAlias "SetFPU(setting, Expression)", "(SetFPU(setting)+(Expression)+RestoreFPU(0))"
   ucAlias "SetOrFPU(setting, Expression)", "(SetOrFPU(setting)+(Expression)+RestoreFPU(0))"
   ucAlias "SetXorFPU(setting, Expression)", "(SetXorFPU(setting)+(Expression)+RestoreFPU(0))"

   ucDefineFunction "Inf() = SetOrFPU(4, 1/0)", 0, %ucDynamic
   ucDefineFunction "NaN() = SetOrFPU(1, 0/0)", 0, %ucDynamic

   ' Hyperbolic functions
   ucDefineFunction "Sinh(x)  = (exp(x) - exp(-x)) / 2"
   ucDefineFunction "Cosh(x)  = (exp(x) + exp(-x)) / 2"
   ucDefineFunction "Tanh(x)  = (exp(2x) - 1) / (exp(2x) + 1)"
   ucDefineFunction "Coth(x)  = (exp(2x) + 1) / (exp(2x) - 1)"
   ucDefineFunction "Sech(x)  = 2 / (exp(x) + exp(-x))"
   ucDefineFunction "Csch(x)  = 2 / (exp(x) - exp(-x))"
   ucDefineFunction "Asinh(x) = Log(x + Sqr(x ^ 2 + 1))"
   ucDefineFunction "Acosh(x) = Log(x + Sqr(x ^ 2 - 1))"
   ucDefineFunction "Atanh(x) = Log((1 + x) / (1 - x)) / 2"
   ucDefineFunction "Acoth(x) = Log((x + 1) / (x - 1)) / 2"
   ucDefineFunction "Asech(x) = Log((1 + Sqr(1 - x ^ 2)) / x)"
   ucDefineFunction "Acsch(x) = Log(1 / x + Sqr(1 + x ^ 2) / Abs(x))"

   ' A few constants needed in some definitions further down
   ucDefineConstant "Pi     = Atn(1) * 4"
   ucDefineConstant "HalfPI = Pi/2"

   ' Other functions
   ucDefineFunction "Sec(x)   = 1 / Cos(x)"
   ucDefineFunction "Csc(x)   = 1 / Sin(x)"
   ucDefineFunction "Cot(x)   = 1 / Tan(x)"
   ucDefineFunction "Asin(x)  = SetOrFPU(4, Atn(x / Sqr(1 - x * x)))" ' +++ make provisions for complex
   ucDefineFunction "Acos(x)  = HalfPI - Asin(x)"
   ucDefineFunction "Log2(x)  = Log(x) / Log(2)"
   ucDefineFunction "Exp2(x)  = 2 ^ x"
   ucDefineFunction "Log10(x) = Log(x) / Log(10)"
   ucDefineFunction "Exp10(x) = 10 ^ x"

   ' Complex operators & functions
   t.ucProperty.FunctionProperties = %ucDynamic + %ucLocked + %ucOverride + %ucOverload + %ucComplex
   ucDefineOperator "+[x] = x.Real + x.Imag*#i", Between_Mult_Pow
   ucDefineOperator "-[x] = (-x.Real) + (-x.Imag)*#i", Between_Mult_Pow

   ucDefineOperator "+[x,y] = x.Real+y.Real + (x.Imag+y.Imag)*#i", ucPrecedence("+")
   ucDefineOperator "*[x,y] = (x.Real*y.Real - x.Imag*y.Imag)+(x.Real*y.Imag + x.Imag*y.Real)*#i", ucPrecedence("*")
   ucDefineOperator "-[x,y] = (x.Real-y.Real) + (x.Imag-y.Imag)*#i", ucPrecedence("-")
   ucDefineOperator "/[x,y] = x.Real*y.Real/(y.Real^2+y.Imag^2) + x.Imag*y.Imag" _
   + "/(y.Real^2+y.Imag^2) + (x.Imag*y.Real/(y.Real^2+y.Imag^2)-x.Real*y.Imag" _
   + "/(y.Real^2+y.Imag^2))*#i", ucPrecedence("/")

   ucDefineFunction "Abs(x) = Sqr(x.Real^2 + x.Imag^2) + 0*#i"
   ucDefineFunction "Sgn(x) = x.Real/Abs(x) + x.Imag/Abs(x)*#i"
   ucDefineFunction "Sin(x) = Sin(x.Real)*Cosh(x.Imag) + Cos(x.Real)*Sinh(x.Imag)*#i"
   ucDefineFunction "Cos(x) = Cos(x.Real)*Cosh(x.Imag) - Sin(x.Real)*Sinh(x.Imag)*#i"
   ucDefineFunction "Phase(x) = iif(x.Imag=0, iif(x.Real>0, 0, pi), Atn(x.Imag/x.Real)) + 0*#i"
   ucDefineFunction "Conj(x)  = x.Real - x.Imag*#i"
   ucDefineFunction "Exp(x)   = Exp(x.Real)*Cos(x.Imag) + Exp(x.Real)*Sin(x.Imag)*#i"
   ucDefineFunction "Imag(x)  = x.Imag + 0*#i"
   ucDefineFunction "Real(x)  = x.Real + 0*#i"

   ucDefineFunction "Sqr(x)   = Sqr(Sqr(x.Real^2 + x.Imag^2)+x.Real)/Sqr(2)" _
   + "+ (Sqr(Sqr(x.Real^2 + x.Imag^2)-x.Real)*iif(x.Imag < 0, -1, 1)/Sqr(2))*#i"
   ucDefineFunction "Sqrt(x)  = Sqr(x.Real+x.Imag*#i)"

   ucDefineFunction "Log(x) = iif(x.Real=0 and x.Imag=0, -Inf," _
   + "Log(x.Real^2 + x.Imag^2)/2)" _
   + "+iif(x.Real=0 and x.Imag=0, 0, iif(x.Real>0 and x.Imag=0, 0," _
   + "iif(x.Imag=0, pi, pi*sgn(x.Imag)/2-Atn(x.Real/x.Imag))))*#i"

   ucDefineFunction "_ZeroToZero() = 1", 0, %ucDynamic+%ucOverwrite
'  ucDefineFunction "_ZeroToZero() = 0", 0, %ucDynamic+%ucOverwrite
'  ucDefineFunction "_ZeroToZero() = NaN", 0, %ucDynamic+%ucOverwrite
'  ucDefineFunction "_ZeroToZero() = SetError(1, '0^0 Error')", 0, %ucDynamic+%ucOverwrite

   ' +++ Can be simplified with SetVar (or other technique)
   ucDefineOperator "^[x,y] = iif(x.Imag = 0, iif(x.Real > 0," _
   + "x.Real^y.Real*Cos(y.Imag*Log(x.Real))," _
   + "iif(x.Real=0 and y.Real=0 and y.Imag=0, _ZeroToZero()," _
   + "(-x.Real)^y.Real*COS(y.Imag*LOG(-x.Real)+pi*y.Real)/EXP(pi*y.Imag)))," _
   + "EXP(y.Imag*ATN(x.Real/x.Imag))*SQR((x.Imag^2+x.Real^2)^y.Real)" _
   + "*COS(pi*y.Real*sgn(x.Imag)/2-y.Real*ATN(x.Real/x.Imag)+y.Imag" _
   + "*LOG(x.Imag^2+x.Real^2)/2)/SQR(EXP(pi*y.Imag*Sgn(x.Imag))))-55+55" _
   + "+iif(x.Imag = 0, iif(x.Real > 0," _
   + "x.Real^y.Real*SIN(y.Imag*LOG(x.Real))," _
   + "iif(x.Real=0 and y.Real=0 and y.Imag=0, 0," _
   + "(-x.Real)^y.Real*SIN(y.Imag*LOG(-x.Real)+pi*y.Real)/EXP(pi*y.Imag)))," _
   + "EXP(y.Imag*ATN(x.Real/x.Imag))*SQR((x.Imag^2+x.Real^2)^y.Real)" _
   + "*SIN(pi*y.Real*sgn(x.Imag)/2-y.Real*ATN(x.Real/x.Imag)+y.Imag" _
   + "*LOG(x.Imag^2+x.Real^2)/2)/SQR(EXP(pi*y.Imag*Sgn(x.Imag)))-55+55)*#i", _
     ucPrecedence("^")

   ucDefineConstant "Byte     = ucType_Integer8"
   ucDefineConstant "Integer  = ucType_Integer16"
   ucDefineConstant "Word     = ucType_Integer16u"
   ucDefineConstant "Long     = ucType_Integer32"
   ucDefineConstant "DWord    = ucType_Integer32u"
   ucDefineConstant "Quad     = ucType_Integer64"
   ucDefineConstant "Currency = ucType_Currency"
   ucDefineConstant "CurrencyX= ucType_CurrencyExt"
   ucDefineConstant "Single   = ucType_FloatSingle"
   ucDefineConstant "Double   = ucType_FloatDouble"
   ucDefineConstant "Extended = ucType_FloatExtended"
   ucDefineConstant "AsciiZ   = ucType_StringLPSTR"
   ucDefineConstant "String   = ucType_StringAnsi"

   ucDefineConstant "Ext = Extended"

   t.ucProperty.FunctionProperties = %ucDynamic + %ucOverload + %ucOverwrite + %ucComplex + %ucRecursion
   t.Context = TempContext
End Sub

'Sub Class_Initialize()

Sub ucEvaluateInit()
   ucInitialized = %ucTrue
   ucGetThreadInfo 0, t0
   ucSetThreadAddr 0, t0
   ucReset t0
'  ucNewThread

'  ucThread = ucNewThread()
'  If ucThread = -1 Then Me = Nothing
End Sub

#Else

Sub ucEvaluateInit()
   Dim t As ucThreadType

   ucInitialized = %ucTrue
   ucGetThreadInfo 0, t
   ReDim ucItem(t.ucProperty.DefSpace.BaseDefinitionSpace _
               +t.ucProperty.DefSpace.DefinitionSpace     _
               *t.ucProperty.DefSpace.MaxThreadCount)     _
               As ucHandleType At t.Handshake.HandleAddr
' +++ Adjust this in other includes.
End Sub

#EndIf
